function [ boxes, boxbin, boxmap ] = ASDgetBox( fname, plotme )
%MSRAgetBox ( filename [, plotme])
%   filename = filename of the image
%   plotme   = plot figure with box, put 2 for making multiple figures
%              put 3 if you want 1-sec delay displaying each picture on
%              single figure
% returns:
%   boxes  = box corners: x1 y1 x2 y2
%            x1 = left, y1 = top, x2 = right, y2 = bottom
%   boxbin = binary of box
%   boxmap = sum of above boxbin
%
% sample usage:
% [a b c] = MSRAgetBox('0_0_147.jpg',1);
%
% a=MSRAgetList; for i=1:10; MSRAgetBox(a{i},3); end
%
%

color = {[1 0 0],[0 1 0],[0 0 1],[1 .5 0],[0 1 1],[1 0 1],[.5 .5 .5],[0 0 0],[1 1 1]};

myFullPath = '/lab/raid/models/saliencyDetection/datasets/ASD/';

if nargin < 2
    plotme = 0;
end

ti = find(fname == '.');
myFname = fname(1:ti-1);
myBmask = imread([myFullPath 'binarymasks/' myFname '.bmp']);
myMask = rgb2gray(myBmask)>127;

boxbin = {myMask};
boxmap = double(myMask);

[ty tx] = ind2sub(size(myMask),find(myMask));
boxes = [min(tx) min(ty) max(tx) max(ty)];


% plotting
if (plotme)
    if plotme == 2; figure; end
    if plotme == 3; pause(1); end
    img = imread([myFullPath 'Imgs/' fname ]);
    if plotme == 4;
        tbb = boxmap;
        tbb = (tbb + 1) / 2;
        img = im2double(img) .* repmat(tbb,[1 1 3]);
    end
    imshow(img);
    for i=1:size(boxes,1)
        s = boxes(i,:);
        ss = [s(1) s(2) s(3)-s(1) s(4)-s(2)];
        %imrect(gca,ss);
        hold on
        if i <= length(color)
            rectangle('position',ss,'edgecolor',color{i});
        else
            rectangle('position',ss,'edgecolor',rand(1,3));
        end
        hold off
    end
    
end

end

