function [ boxes, boxbin, boxmap ] = MSRAgetBox( fname, plotme )
%MSRAgetBox ( filename [, plotme])
%   filename = filename of the image
%   plotme   = plot figure with box, put 2 for making multiple figures
%              put 3 if you want 1-sec delay displaying each picture on
%              single figure
% returns:
%   boxes  = box corners: x1 y1 x2 y2
%            x1 = left, y1 = top, x2 = right, y2 = bottom
%   boxbin = binary of box
%   boxmap = sum of above boxbin
%
% sample usage:
% [a b c] = MSRAgetBox('0_0_147.jpg',1);
%
% a=MSRAgetList; for i=1:10; MSRAgetBox(a{i},3); end
%
%

color = {[1 0 0],[0 1 0],[0 0 1],[1 .5 0],[0 1 1],[1 0 1],[.5 .5 .5],[0 0 0],[1 1 1]};

myFullPath = '/lab/raid/models/saliencyDetection/datasets/MSRA/';

if nargin < 2
    plotme = 0;
end

% this to avoid reloading the file multiple times
global MSRAdatasets;
if isempty(MSRAdatasets)
    load('MSRAtags.mat');
    MSRAdatasets{1} = picname;
    MSRAdatasets{2} = picsize;
    MSRAdatasets{3} = picbox;
end

% parsing
zz = strcmp(MSRAdatasets{1},fname);
boxes = MSRAdatasets{3}{zz};
imgsize = MSRAdatasets{2}(zz,:);
boxmap = zeros(imgsize)';
boxbin = cell(size(boxes,1),1);
for i=1:size(boxes,1)
    s = boxes(i,:);
    c = [s(1) s(3) s(3) s(1)];
    r = [s(2) s(2) s(4) s(4)];
    BW=roipoly(boxmap,c,r);
    boxmap = boxmap + BW;
    boxbin{i} = BW;
end

% plotting
if (plotme)
    if plotme == 2; figure; end
    if plotme == 3; pause(1); end
    idx=find(zz);
    fldr = floor((idx-1)/500); % they only have 500 image on each folder
    img = imread([myFullPath 'ImageB/Image/' num2str(fldr) '/' fname ]);
    imshow(img);
    for i=1:size(boxes,1)
        s = boxes(i,:);
        ss = [s(1) s(2) s(3)-s(1) s(4)-s(2)];
        %imrect(gca,ss);
        hold on
        rectangle('position',ss,'edgecolor',color{i});
        hold off
    end
    
end

end




