clear all
close all
clc

myTic = tic;

generatePath;

% --------------------------------------------
dims = [50 50]; sigma = 10; % gaussian standard deviation in pixels
P = [round(dims(1)/2) round(dims(2)/2)];
[Xm Ym] = meshgrid(-P(2):P(2), -P(1):P(1)); s = sigma ;
gauss = exp(-((( Xm.^2)+( Ym.^2)) ./ (2* s^2)));

myPath = '/lab/raid/models/saliencyDetection/';



% Score type:
% 1  = AUC Judd version
% 2  = AUC random, our version
% 3  = AUC shuffle, our version
% 4  = smart chance (control), eye fixation from other random image
% 5  = full random (control), AUC should be 0.5
% 6  = normal random (control), a normal distribution of eye fixation
% 7  = NSS
% 8  = CC
scoringType = 3;

gaussSize=[ 10 30 50];

% param for #3, percentage of max vote
thresholdedVotePercentage = .5;

datasetUsed = { ...
    %     'ASD'
    %     'MSRA'
    % %     'SED'
    % %     'SED2'
    % %     'SOD'
    'JOV2008d'
    };


% Info: 'modelName' % ModelAcronym . Code . Size . Citation . Dataset avail
% * code: M=matlab only, MC=matlab+C, C=C/C++, S=provided
modelUsed = { ...
    %     % models that we have (have code) = all datasets calculated
    %     'Goferman'   % CA      . M  . max 250 . Gofferman et.al ICVS 2008 . All
    %     'EDS'        % EDS     . MC . orig    . Rosin et.al PRx/2009      . All
    %     'CBsal'      % CBSAL   . MC . orig    . Jiang et.al BMVC 2011     . All
    %     'Materias'   % JL      . M  . orig    . Jian Li BMVC 2011         . All
    %     'salLiu'     % LIU     . MC . max 200 . Liu et al. CVPR 2007      . All
    %     'SVO'        % SVO     . MC . orig    . Chang et al. ICCV 2011    . All
    %     'MSSS'       % MSSS    . M  . orig    . Achanta ICIP 2008         . All
    %     'AC'         % AC      . M  . orig    . Achanta ICVS 2008         . All
    %     'FTS'        % FT      . M  . orig    . Achanta CVPR 2009         . All
    %     'MichalGazit'% GAZIT   . M  . orig    . Gazit et.al ECCV2010      . All
    %     % some precomputed models
    %     %%%%%%'CA' % ASD only- This is Gofferman, Just keep this.
    %     %%%%%%'RCC' % Thresholded version of RC, ignore this
    %     'MZ'         % MZ      . S  . orig    . Ma and Zhang 2003         . ASD only
    %     'Valenti'    % VALENTI . S  . orig    . Valenti et.al ICCV 2009   . ASD and MSRA only
    %     'DAKlein'    % BITS    . S  . orig    . Klein & Frintrop ICCV 2011. All
    %     'PMehrani'   % MV      . S  . orig    . Mehrani et.al BMVC 2010   . ASD and SED1 only
    %     'LiuICIP'    % LIU-ICIP. S  . orig    . Liu et.al ICIP 2010       . ASD only
    %     'LiuIETIP'   % LIU-TiP . S  . orig    . Liu et.al TiP 2011        . ASD only
    %     'MengW'      % MENG    . S  . orig    .                           . ASD only
    %     %%precomputed on windoze machine
    %     %%%%% 'FT' % = FTS, ignore this model
    %     %%%%% 'SR' % This is the same as HouCVPR
    %     'HC'         % HC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    %     'LC'         % LC      . C  . orig    . Zhai and Shah ACM-M 2006  . All
    %     'RC'         % RC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    %     %partial result, maybe biased due to train&test on same dataset but different files
    %     'JiaLiSal'   % JIA-LI  . S  . 1/16    . Jia Li et.al IJCV 2010    . Partial ASD and MSRA
    
    
    %     % Saliency modeling models (for fixation prediction)
    %     'SAIM'       % AIM     . M  . 1/2     .
    %     'SHouCVPR'   % SR      . M  . 64x64   . SRM % SKIP This as it is the same as SR
    %     'SHouNIPS'   % ICL     . M  . orig    .
    %     'SSUN'       % SUN     . M  . 1/2 - N .
        'SAWS'       % AWS     . M  . 1/2     .
    %     'SPQFT'      % PQFT    . M  . 400x400 .
    %     'SGBVS'      % GBVS    . M  . orig    .have problem when the image is under 128x128, same as GB
        'SITTI'      % ITTI    . C  . 1/16    . Itti model - same as IT
        'SITTI98'    % ITTI98  . C  . 1/16    . Maxnorm
    %     'SSEO'       % SDSR    . M  . orig    .
    %     'SJudd'      % Judd    . M  . orig    .
    %
    %
    %     % MAP map (Mean Annotation Map)
    %     'MEP' % Mean Annotation Map . 500x500
    %'IO'  % >>> Inter observer  . orig
        'Gauss'
    
    %     % segmenters, not salient object detections
    %     'SRS1' % SRS . Christian's model, SRS# where # is the #-th salient point
    %     'SRS2'
    %     'SRS3'
    %     'SRS4'
    %     'SRS5'
    %     'Mishra' % MISHRA .
    %
    %     % some combination maps, 3 best
    %     'CMAPmlin' % mult linear
    %     'CMAPplin' % plus linear
    %     'CMAPplog' % plus log(x/2)
    %     'CMAPpexp' % plus exp(x^3)
    %     % 2 best
    %     'CMAP2mlin' % mult linear
    %     'CMAP2plin' % plus linear
    %     'CMAP2plog' % plus log(x/2)
    %     'CMAP2pexp' % plus exp(x^3)
    
    % JOV
        'JOV2008d_case1'
    
    };

colorCode={'m','c','r','g','b','k'};
lineCode ={'-','--',':','-.'};
theCodes=cell(length(colorCode)*length(lineCode),1);
for i=1:length(colorCode)
    for j=1:length(lineCode)
        theCodes{(i-1)*length(lineCode)+j} = [colorCode{i} lineCode{j}];
    end
end

myGauss=fspecial('gaussian',200,10);
myGauss=myGauss-min(min(myGauss));
myGauss=myGauss./max(max(myGauss));

% loop dataset
for ds=1:length(datasetUsed)
    eval(['imName = ' datasetUsed{ds} 'getList;'])
    
    scoreROCa = cell(length(imName),length(modelUsed),length(gaussSize)); %
    scoreROCt = cell(length(imName),length(modelUsed),length(gaussSize)); %tpp?
    scoreROCf = cell(length(imName),length(modelUsed),length(gaussSize)); %fpp?
    
    % HAX
    load('/lab/raid/models/saliencyDetection/datasets/JOV2008d_export/dataAna/fixMaps.mat');
    IO=load('/lab/raid/models/saliencyDetection/datasets/JOV2008d_export/dataAna/fixMapsPerSubj.mat');
    % loop image
    for imNo = 1:length(imName)
        %eval(['[imBox imEachGT imAllGT] = ' datasetUsed{ds} 'getBox(imName{imNo});'])
        
        % loop models
        for mm=1:length(modelUsed)
            %printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
            for gg=1:length(gaussSize)
                myGauss=fspecial('gaussian',200,gaussSize(gg));
                myGauss=myGauss-min(min(myGauss));
                myGauss=myGauss./max(max(myGauss));
                
                if strcmp(modelUsed{mm},'IO') % Inter observer model
                    if strcmp(datasetUsed{ds},'ASD'); continue; end % ASD doesnt have IO
                    printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
                    
                    mySubjNo=size(IO.fixMaps,1);
                    tROCa = cell(mySubjNo,1);
                    tROCt = cell(mySubjNo,1);
                    tROCf = cell(mySubjNo,1);
                    
                    % inter observer: take n-th, compare with 1 ~ n-1 stuffs
                    for me = 1:mySubjNo
                        idx=1:mySubjNo;
                        idx=idx(idx~=me);
                        otherFix=sum(cat(3,IO.fixMaps{idx,imNo}),3);
                        SM = conv2(double(otherFix),myGauss,'same');
                        SM = imresize(SM,size(fixMaps{imNo}),'nearest');
                        SM = SM + imresize(myGauss,size(fixMaps{imNo}),'nearest');
                        human=IO.fixMaps{me,imNo};
                        switch(scoringType)
                            case 1
                                % AUC ROC type 1 : Judd's score calculation
                                [a t f] = calcAUCjudd(SM,fixMaps{imNo});
                                tROCa{me} = a;
                                tROCt{me} = imresize(t,[1000 1],'nearest');
                                tROCf{me} = imresize(f,[1000 1],'nearest');
                            case 2
                                % AUC ROC type 2 : Ours random
                                [a] = calcAUCscore(SM,fixMaps{imNo});
                                tROCa{me} = mean(a);
                            case 3
                                % AUC ROC type 3 : Ours shuffled
                                idx=1:length(fixMaps);
                                idx=idx(idx~=imNo);
                                otherFix=sum(cat(3,fixMaps{idx}),3);
                                [a] = calcAUCscore(SM,fixMaps{imNo},otherFix);
                                tROCa{me} = mean(a);
                            case 4
                                % random test case ( control? )
                                idx=1:length(fixMaps);
                                idx=idx(idx~=imNo);
                                idx=idx(randperm(length(idx)));
                                [a t f] = calcAUCjudd(SM,fixMaps{idx(1)});
                                tROCa{me} = a;
                                tROCt{me} = imresize(t,[1000 1],'nearest');
                                tROCf{me} = imresize(f,[1000 1],'nearest');
                            case 5
                                
                                % full random test case ( uniform random, AUC must around 0.5 )
                                x=randi(1024,1,1000);
                                y=randi(768,1,1000);
                                fullRND=false(size(fixMaps{imNo} ) );
                                idx=sub2ind([768 1024],y,x);
                                fullRND(idx)=1;
                                [a t f] = calcAUCjudd(SM,fullRND);
                                tROCa{me} = a;
                                tROCt{me} = imresize(t,[1000 1],'nearest');
                                tROCf{me} = imresize(f,[1000 1],'nearest');
                            case 6
                                % full random test case ( normal/gauss random )
                                x=round((randn(1,1000) ./ 4 .* 512) + 512);
                                y=round((randn(1,1000) ./ 4 .* 384) + 384);
                                x=min(max(x,1),1024);
                                y=min(max(y,1),768);
                                fullRND=false(size(fixMaps{imNo} ) );
                                idx=sub2ind([768 1024],y,x);
                                fullRND(idx)=1;
                                [a t f] = calcAUCjudd(SM,fullRND);
                                tROCa{me} = a;
                                tROCt{me} = imresize(t,[1000 1],'nearest');
                                tROCf{me} = imresize(f,[1000 1],'nearest');
                            case 7
                                % NSS
                                [a] = calcNSSscore(SM,fixMaps{imNo});
                                tROCa{me} = mean(a);
                            case 8
                                % CC
                                tROCa{me} = calcCCscore(SM,fixMaps{imNo});
                        end
                    end
                    scoreROCa{imNo,mm,gg} = mean(horzcat(tROCa{:}));
                    scoreROCt{imNo,mm,gg} = mean(horzcat(tROCt{:}),2);
                    scoreROCf{imNo,mm,gg} = mean(horzcat(tROCf{:}),2);
                else
                    if strcmp(modelUsed{mm},'MEP')
                        theName = [myPath 'output/MEP_' datasetUsed{ds} '.mat'];
                    else
                        theName = [myPath 'output/' modelUsed{mm} '/' datasetUsed{ds} '/' rmExt(imName{imNo}) '.mat'];
                    end
                    if exist(theName,'file') || strcmp(modelUsed{mm},'Gauss')
                        if strcmp(modelUsed{mm},'Gauss')
                            SM = myGauss;
                            SM = imresize(SM,size(fixMaps{imNo}),'nearest');
                        else
                        load(theName);
                        %SM = conv2(SM,myGauss,'same');
                        if size(SM,3)>1; SM=rgb2gray(SM); end
                        SM = imresize(SM,size(fixMaps{imNo}),'nearest');
                        SM = SM -  min(min(SM));
                        SM = SM ./ max(max(SM));
                        SM = SM + imresize(myGauss,size(fixMaps{imNo}),'nearest');
                        end
                        printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
                        
%                         subplot(1,3,gg)
%                         imshow(SM,[])
%                         title(['Model=' modelUsed{mm} ' gaussSize=' num2str(gaussSize(gg))])
%                         if gg==3
%                             pause(.1)
%                         end
                        
                        switch(scoringType)
                            case 1
                                % AUC ROC type 1 : Judd's score calculation
                                [a t f] = calcAUCjudd(SM,fixMaps{imNo});
                                scoreROCa{imNo,mm,gg} = a;
                                scoreROCt{imNo,mm,gg} = imresize(t,[1000 1],'nearest');
                                scoreROCf{imNo,mm,gg} = imresize(f,[1000 1],'nearest');
                            case 2
                                % AUC ROC type 2 : Ours random
                                [a] = calcAUCscore(SM,fixMaps{imNo});
                                scoreROCa{imNo,mm,gg} = mean(a);
                            case 3
                                % AUC ROC type 3 : Ours shuffled
                                idx=1:length(fixMaps);
                                idx=idx(idx~=imNo);
                                otherFix=sum(cat(3,fixMaps{idx}),3);
                                [a] = calcAUCscore(SM,fixMaps{imNo},otherFix);
                                scoreROCa{imNo,mm,gg} = mean(a);
                            case 4
                                % random test case ( control? )
                                idx=1:length(fixMaps);
                                idx=idx(idx~=imNo);
                                idx=idx(randperm(length(idx)));
                                [a t f] = calcAUCjudd(SM,fixMaps{idx(1)});
                                scoreROCa{imNo,mm,gg} = a;
                                scoreROCt{imNo,mm,gg} = imresize(t,[1000 1],'nearest');
                                scoreROCf{imNo,mm,gg} = imresize(f,[1000 1],'nearest');
                            case 5
                                
                                % full random test case ( uniform random, AUC must around 0.5 )
                                x=randi(1024,1,1000);
                                y=randi(768,1,1000);
                                fullRND=false(size(fixMaps{imNo} ) );
                                idx=sub2ind([768 1024],y,x);
                                fullRND(idx)=1;
                                [a t f] = calcAUCjudd(SM,fullRND);
                                scoreROCa{imNo,mm,gg} = a;
                                scoreROCt{imNo,mm,gg} = imresize(t,[1000 1],'nearest');
                                scoreROCf{imNo,mm,gg} = imresize(f,[1000 1],'nearest');
                            case 6
                                % full random test case ( normal/gauss random )
                                x=round((randn(1,1000) ./ 4 .* 512) + 512);
                                y=round((randn(1,1000) ./ 4 .* 384) + 384);
                                x=min(max(x,1),1024);
                                y=min(max(y,1),768);
                                fullRND=false(size(fixMaps{imNo} ) );
                                idx=sub2ind([768 1024],y,x);
                                fullRND(idx)=1;
                                [a t f] = calcAUCjudd(SM,fullRND);
                                scoreROCa{imNo,mm,gg} = a;
                                scoreROCt{imNo,mm,gg} = imresize(t,[1000 1],'nearest');
                                scoreROCf{imNo,mm,gg} = imresize(f,[1000 1],'nearest');
                            case 7
                                % NSS
                                [a] = calcNSSscore(SM,fixMaps{imNo});
                                scoreROCa{imNo,mm,gg} = mean(a);
                            case 8
                                % CC
                                [scoreROCa{imNo,mm,gg}] = calcCCscore(SM,fixMaps{imNo});
                        end
                        clear SM;
                        
                    else
                        printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm},'not found')
                    end
                end
                
            end
        end
    end
    if length(modelUsed) == 1
        save(['score/EyeCB_' datasetUsed{ds} '_' date '__' modelUsed{1} '_' num2str(scoringType) '.mat'],'scoreROCa','scoreROCt','scoreROCf','modelUsed')
    else
        save(['score/EyeCB_' datasetUsed{ds} '_' date '__modelSegmenters_' num2str(scoringType) '.mat'],'scoreROCa','scoreROCt','scoreROCf','modelUsed')
    end
end
% exit