% clear all
% close all
% clc

myTic = tic;

generatePath;

% --------------------------------------------
dims = [50 50]; sigma = 10; % gaussian standard deviation in pixels
P = [round(dims(1)/2) round(dims(2)/2)];
[Xm Ym] = meshgrid(-P(2):P(2), -P(1):P(1)); s = sigma ;
gauss = exp(-((( Xm.^2)+( Ym.^2)) ./ (2* s^2)));

myPath = '/lab/raid/models/saliencyDetection/';

% Score mode,
% 1 = average of individual subject's tag(s)
% 2 = highest vote
% 3 = thresholded vote
scoreMode = 2;

% Comparison (Groundtruth vs salmaps) type
% 1 = object tight segmented
% 2 = bounding boxes / edges of the ground truth
% 3 = BW fill
comparisonType = 1;

% IO type:
% 1 = leave one out method
% 2 = one to one
IOtype=1;

% param for #3, percentage of max vote
thresholdedVotePercentage = .5;

datasetUsed = { ...
%     'ASD'
%     'MSRA'
%     'SED'
    'SED2'
%     'SOD'
%     'ImgSal'
%     'Bruce'
    };


% Info: 'modelName' % ModelAcronym . Code . Size . Citation . Dataset avail
% * code: M=matlab only, MC=matlab+C, C=C/C++, S=provided
modelUsed = { ...
    % models that we have (have code) = all datasets calculated
%     'Goferman'   % CA      . M  . max 250 . Gofferman et.al ICVS 2008 . All
%     'EDS'        % EDS     . MC . orig    . Rosin et.al PRx/2009      . All
%     'CBsal'      % CBSAL   . MC . orig    . Jiang et.al BMVC 2011     . All
%     'Materias'   % JL      . M  . orig    . Jian Li BMVC 2011         . All
%     'salLiu'     % LIU     . MC . max 200 . Liu et al. CVPR 2007      . All
%     'SVO'        % SVO     . MC . orig    . Chang et al. ICCV 2011    . All
%     'MSSS'       % MSSS    . M  . orig    . Achanta ICIP 2008         . All
%     'AC'         % AC      . M  . orig    . Achanta ICVS 2008         . All
%     'FTS'        % FT      . M  . orig    . Achanta CVPR 2009         . All
%     'MichalGazit'% GAZIT   . M  . orig    . Gazit et.al ECCV2010      . All
    % some precomputed models
    %%%%%%'CA' % ASD only- This is Gofferman, Just keep this.
    %%%%%%'RCC' % Thresholded version of RC, ignore this
%     'MZ'         % MZ      . S  . orig    . Ma and Zhang 2003         . ASD only
%     'Valenti'    % VALENTI . S  . orig    . Valenti et.al ICCV 2009   . ASD and MSRA only
%     'DAKlein'    % BITS    . S  . orig    . Klein & Frintrop ICCV 2011. All
%     'PMehrani'   % MV      . S  . orig    . Mehrani et.al BMVC 2010   . ASD and SED1 only
%     'LiuICIP'    % LIU-ICIP. S  . orig    . Liu et.al ICIP 2010       . ASD only
%     'LiuIETIP'   % LIU-TiP . S  . orig    . Liu et.al TiP 2011        . ASD only
%     'MengW'      % MENG    . S  . orig    .                           . ASD only
    %%precomputed on windoze machine
    %%%%% 'FT' % = FTS, ignore this model
    %%%%% 'SR' % This is the same as HouCVPR
%     'HC'         % HC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
%     'LC'         % LC      . C  . orig    . Zhai and Shah ACM-M 2006  . All
%     'RC'         % RC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    %partial result, maybe biased due to train&test on same dataset but different files
%     'JiaLiSal'   % JIA-LI  . S  . 1/16    . Jia Li et.al IJCV 2010    . Partial ASD and MSRA
%     
%     
%     % Saliency modeling models (for fixation prediction)
    'SAIM'       % AIM     . M  . 1/2     .
    'SHouCVPR'   % SR      . M  . 64x64   . SRM % SKIP This as it is the same as SR
    'SHouNIPS'   % ICL     . M  . orig    .
    'SSUN'       % SUN     . M  . 1/2 - N .
    'SAWS'       % AWS     . M  . 1/2     .
    'SPQFT'      % PQFT    . M  . 400x400 .
    'SGBVS'      % GBVS    . M  . orig    .have problem when the image is under 128x128, same as GB
    'SITTI'      % ITTI    . C  . 1/16    . Itti model - same as IT
    'SITTI98'    % ITTI98  . C  . 1/16    . Maxnorm
    'SSEO'       % SDSR    . M  . orig    .
%     'SJudd'      % Judd    . M  . orig    .
%     
%     
%     % MAP map (Mean Annotation Map)
%     'MEP' % Mean Annotation Map . 500x500
%     'IO'  % >>> Inter observer  . orig
    'objSize'
%     
%     % segmenters, not salient object detections
%     'SRS1' % SRS . Christian's model, SRS# where # is the #-th salient point
%     %     'SRS2'
%     %     'SRS3'
%     %     'SRS4'
%     %     'SRS5'
%     'Mishra' % MISHRA .
%     
%     % some combination maps, 3 best
%     'CMAPmlin' % mult linear
%     'CMAPplin' % plus linear
%     'CMAPplog' % plus log(x/2)
%     'CMAPpexp' % plus exp(x^3)
% %     % 2 best
% %     'CMAP2mlin' % mult linear
% %     'CMAP2plin' % plus linear
% %     'CMAP2plog' % plus log(x/2)
% %     'CMAP2pexp' % plus exp(x^3)

    };

colorCode={'m','c','r','g','b','k'};
lineCode ={'-','--',':','-.'};
theCodes=cell(length(colorCode)*length(lineCode),1);
for i=1:length(colorCode)
    for j=1:length(lineCode)
        theCodes{(i-1)*length(lineCode)+j} = [colorCode{i} lineCode{j}];
    end
end

% loop dataset
for ds=1:length(datasetUsed)
    eval(['imName = ' datasetUsed{ds} 'getList;'])
    
%     scorePRnew= cell(length(imName),length(modelUsed)); % new
%     scorePRpr = cell(length(imName),length(modelUsed)); %precision
%     scorePRre = cell(length(imName),length(modelUsed)); %recall
%     scoreROCt = cell(length(imName),length(modelUsed)); %tpp?
%     scoreROCf = cell(length(imName),length(modelUsed)); %fpp?
    scoreHit = cell(length(imName),length(modelUsed));
    avgSMscore = cell(length(imName),length(modelUsed),5);
    
    avgSMsel = zeros(length(imName),length(modelUsed));
    avgSMnon = zeros(length(imName),length(modelUsed));
    avgSMbgd = zeros(length(imName),length(modelUsed));
    avgSMlge = zeros(length(imName),length(modelUsed));
    avgSMsml = zeros(length(imName),length(modelUsed));
    
    hitSMsel = zeros(length(imName),length(modelUsed));
    hitSMnon = zeros(length(imName),length(modelUsed));
    hitSMbgd = zeros(length(imName),length(modelUsed));
    
    rndSel = zeros(length(imName),1);
    rndNon = zeros(length(imName),1);
    
    
    ranDsel = zeros(length(imName),100);
    ranDnon = zeros(length(imName),100);
    ranDbgd = zeros(length(imName),100);
    
    subjSMsel = zeros(length(imName),length(modelUsed));
    subjSMnon = zeros(length(imName),length(modelUsed));
    subjSMbgd = zeros(length(imName),length(modelUsed));

    %load('/lab/raid/models/saliencyDetection/salJudgeData/exp1Result.mat')
    load('/lab/raid/models/saliencyDetection/salJudgeData/exp1_v2.mat')
    
    % loop image
    for imNo = 1:length(imName)
        %eval(['[imBox imEachGT imAllGT imEachGTZ] = ' datasetUsed{ds} 'getBox(imName{imNo});'])
        %imAllGT=subjAgreement{imNo};
        % loop models
        for mm=1:length(modelUsed)
            %printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
            if strcmp(modelUsed{mm},'IO') % Inter observer model
                if strcmp(datasetUsed{ds},'ASD'); continue; end % ASD doesnt have IO
                printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
%                 
%                 
%                 if IOtype==2 % one to one
%                 tPR = cell(length(imEachGT),length(imEachGT));
%                 tROC= cell(length(imEachGT),length(imEachGT));
% %                 tPRT= cell(length(imEachGT),length(imEachGT),2);
%                 % inter observer: take n-th, compare with 1 ~ n-1 stuffs
%                 for me = 1:length(imEachGT)
%                     SM = double(imEachGT{me});
%                     SM = conv2(SM,gauss,'same');
%                     for mu = me+1:length(imEachGT)
%                         if me == mu; continue; end
%                         % calc PR + ROC
%                         [tPR{me,mu} tROC{me,mu}] = calcPR(SM,imEachGT{mu},comparisonType);
%                     end
%                 end
%                 w=vertcat(tPR{:});
%                 a=w(1:2:end,:); %precision
%                 scorePRpr{imNo,mm}=mean(a,1);
%                 b=w(2:2:end,:); %recall
%                 scorePRre{imNo,mm}=mean(b,1);
%                     
%                 
%                 w=vertcat(tROC{:});
%                 a=w(1:2:end,:); %tpp
%                 scoreROCt{imNo,mm}=mean(a,1);
%                 b=w(2:2:end,:); %fpp
%                 scoreROCf{imNo,mm}=mean(b,1);
%                 else % leave one out --------------------------------
%                 tPR = cell(length(imEachGT),1);
%                 tROC= cell(length(imEachGT),1);
%                 for me = 1:length(imEachGT)
%                     idx=1:length(imEachGT);
%                     idx=idx(idx~=me);
%                     otherFix=sum(cat(3,imEachGT{idx}),3);
%                     %SM = conv2(double(otherFix),gauss,'same');
%                     SM=otherFix;
%                     [tPR{me} tROC{me}] = calcPR(SM,imEachGT{me},comparisonType);
%                 end
%                                 w=vertcat(tPR{:});
%                 a=w(1:2:end,:); %precision
%                 scorePRpr{imNo,mm}=mean(a,1);
%                 b=w(2:2:end,:); %recall
%                 scorePRre{imNo,mm}=mean(b,1);
%                     
%                 
%                 w=vertcat(tROC{:});
%                 a=w(1:2:end,:); %tpp
%                 scoreROCt{imNo,mm}=mean(a,1);
%                 b=w(2:2:end,:); %fpp
%                 scoreROCf{imNo,mm}=mean(b,1);
% 
%                 end

            else
            if strcmp(modelUsed{mm},'MEP')
                theName = [myPath 'output/MEP_' datasetUsed{ds} '.mat'];
            else
                theName = [myPath 'output/' modelUsed{mm} '/' datasetUsed{ds} '/' rmExt(imName{imNo}) '.mat'];
            end
            if exist(theName,'file') || ...
                    strcmp(modelUsed{mm},'objSize')
                
                if strcmp(modelUsed{mm},'objSize')
                    a=mySegObj{imNo,1};
                    b=mySegObj{imNo,2};
                    if mean2(a) > mean2(b)
                        SM = a;
                    else
                        SM = b;
                    end
                    
%                     numObjSize=zeros(length(imEachGTZ),1);
%                     for ki=1:length(imEachGTZ)
%                         numObjSize(ki)=sum(sum(imEachGTZ{ki}));
%                     end
%                     [a b]=max(numObjSize);
%                     SM = double(imEachGTZ{b});

                else
                    load(theName);
                end
                printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
                
                SM = double(SM);
                if size(SM,3)>1
                    SM = rgb2gray(SM);
                end
                SM = imresize(SM,size(mySegObj{imNo,1}),'nearest');
                SM = SM - min(min(SM));
                SM = SM ./ max(max(SM));
%                 subplot(1,2,1);imshow(SM,[])
                
                %%% Find which tag object gives higher average
%                 numObjSize=zeros(length(imEachGTZ),1);
%                 for ki=1:length(imEachGTZ)
% %                     numObjSize(ki)=mean2(SM .* imEachGTZ{ki})./mean2(imEachGTZ{ki});
%                     numObjSize(ki)=max(max(SM .* imEachGTZ{ki})); %find the max inside which object
%                 end
%                 [a b]=max(numObjSize);
%                 SM = double(imEachGTZ{b});
%                 subplot(1,2,2);imshow(SM,[])
%                 pause
                
% %                 SM = SM == 1;
% %                 
% %                 % what if we have multiple peaks?
% %                 while sum(sum(SM)) > 1 % threshold?
% %                     SM = double(SM);
% %                     SM = conv2(SM,fspecial('gaussian',50,10),'same');
% %                     SM = SM - min(min(SM));
% %                     SM = SM ./ max(max(SM));
% %                     SM = SM == 1;
% %                 end
                
%                 maxVote = max(max(imAllGT));
%                 tVote = imAllGT == maxVote;
%                 
%                 combi = SM & tVote;
%                 imshow(SM)
%                 scoreHit{imNo,mm}= mean2(combi) > 0;

                

                
                
                %%% The score system %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                za=mySegObj{imNo,1}; % segmentation obj A
                zb=mySegObj{imNo,2};
                
                % try to expand the segmentation
                %se = strel('square',9);
                %za = imdilate(za,se);
                %zb = imdilate(zb,se);
                
                xa=myObjCount(imNo,1); % num ppl chose obj A
                %xb=myObjCount(imNo,2);
                xb=70-xa;
                peopleChoose = xa > xb;
                largest = mean2(za) > mean2(zb);
                mySize = size(SM);
                mySize = mySize(1)*mySize(2);
                SMa = SM .* za;
                SMb = SM .* zb;
                SMc = SM .* ~(za | zb);
                % average
                %avgSMa = sum(sum(SMa)) ./ sum(sum(za));% ./(sum(sum(za))/mySize);
                %avgSMb = sum(sum(SMb)) ./ sum(sum(zb));%./(sum(sum(zb))/mySize);
                % maximum
                avgSMa = max(max(SMa));
                avgSMb = max(max(SMb));
                avgSMc = max(max(SMc));
                % new
                %avgSMa = max(max(SMa))./(sum(sum(za))/mySize);
                %avgSMb = max(max(SMb))./(sum(sum(zb))/mySize);
                
                subjSMsel(imNo,mm)=avgSMa*xa + avgSMb*xb;
                subjSMnon(imNo,mm)=avgSMa*xb + avgSMb*xa;
                subjSMbgd(imNo,mm)=avgSMc;
                
                SMchoose = avgSMa > avgSMb;
                if peopleChoose == SMchoose
                    scoreHit{imNo,mm} = 1;
                else
                    scoreHit{imNo,mm} = 0;
                end
                
                avgSMscore{imNo,mm,1}=avgSMa;
                avgSMscore{imNo,mm,2}=avgSMb;
                avgSMscore{imNo,mm,3}=avgSMc;
                
                %%% Accuracy hit %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                mySMbin = SM .* (za | zb); % binary decision
                mySMbin = mySMbin ./ max(max(mySMbin));
                
                %%% adding SM OR Largest
                a=mySegObj{imNo,1};
                b=mySegObj{imNo,2};
                if mean2(a) > mean2(b)
                    SMlge = a;
                else
                    SMlge = b;
                end
                
                
                %%% Random system %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %ranDsel = zeros(length(imName),100);
                %ranDnon = zeros(length(imName),100);
                %ranDbgd = zeros(length(imName),100);
                %segmentation: za zb
                %num ppl sel:  xa xb
                
                %loop 100 times
                for ri = 1:100
                    % random point
                    R1=randi(size(SM,1));
                    R2=randi(size(SM,2));
                    
                    %check obj A
                    if za(R1,R2)==1
                        ranDsel(imNo,ri)=xa;
                        ranDnon(imNo,ri)=xb;
                        ranDbgd(imNo,ri)=0;
                    elseif zb(R1,R2)==1 %obj B
                        ranDsel(imNo,ri)=xb;
                        ranDnon(imNo,ri)=xa;
                        ranDbgd(imNo,ri)=0;
                    else
                        ranDsel(imNo,ri)=0;
                        ranDnon(imNo,ri)=0;
                        ranDbgd(imNo,ri)=70;
                    end
                end
                %pick a random point
                %if on the left
                %    lefCounter = lefCounter + za (num subjects chose left)
                %else on right 
                %    RightlCounter = RightlCounter + zb (num subjects chose Rightl)
                %else
                %    BGCounter = BGCounter + 70 (num subjects chose BG)
                %end                

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                mySMhit = (mySMbin == 1) | SMlge; %%% <---------------------
                %mySMhit = (mySMbin == 1); %%%%%% Without Largest, binary mode
                %mySMhit = (SM == 1); %%%%%% Raw Saliency
                %if mean2(~(za | zb) & mySMhit) > 0
                %    hitSMbgd(imNo,mm)=1;
%                 end
                %else
                if mean2((za | zb) & mySMhit) > 0
                if peopleChoose % object a is chosen(higher)
                    if mean2(za & mySMhit) > 0 % hit on A
                        %hitSMsel(imNo,mm)=1;
                        hitSMsel(imNo,mm)=xa/70;
                        hitSMnon(imNo,mm)=xb/70;
                    else % miss
                        hitSMsel(imNo,mm)=xb/70;
                        hitSMnon(imNo,mm)=xa/70;
                    end
%                     if mean2(zb & mySMhit) == 0
%                         %hitSMnon(imNo,mm)=1;
%                         hitSMnon(imNo,mm)=xb/70;
%                     else
%                         hitSMnon(imNo,mm)=xa/70;
%                     end
                    rndSel(imNo) = sum(sum(za)) / (size(za,1)*size(za,2));
                    rndNon(imNo) = sum(sum(zb)) / (size(zb,1)*size(zb,2));
                else % object b is chosen (higher)
                    if mean2(zb & mySMhit) > 0
                        %hitSMsel(imNo,mm)=1;
                        hitSMsel(imNo,mm)=xb/70;
                        hitSMnon(imNo,mm)=xa/70;
                    else
                        hitSMsel(imNo,mm)=xa/70;
                        hitSMnon(imNo,mm)=xb/70;
                    end
%                     if mean2(za & mySMhit) == 0
%                         %hitSMnon(imNo,mm)=1;
%                         hitSMnon(imNo,mm)=xa/70;
%                     else
%                         hitSMnon(imNo,mm)=xb/70;
%                     end
                    rndSel(imNo) = sum(sum(zb)) / (size(zb,1)*size(zb,2));
                    rndNon(imNo) = sum(sum(za)) / (size(za,1)*size(za,2));
                end
                else
                    hitSMbgd(imNo,mm)=1;
                end

                
%                 if strcmp(modelUsed{mm},'objSize')
%                     
%                     a=mySegObj{imNo,1};
%                     b=mySegObj{imNo,2};
%                     if mean2(a) > mean2(b)
%                         aa=a;
%                         bb=b;
%                     else
%                         aa=b;
%                         bb=a;
%                     end
%                     % aa=obj with bigger size, bb= lower size
%                     
%                     if peopleChoose % sel = most ppl select this obj
%                         sel = aa;
%                         non = bb;
%                         psel=xa;
%                         pnon=xb;
%                     else
%                         sel = bb;
%                         non = aa;
%                         psel=xb;
%                         pnon=xa;
%                     end
%                     A = sum(sum(sel))/mySize;
%                     B = sum(sum(non))/mySize;
%                     
%                     %avgSMsel(imNo,mm)=((A/(sum(sum(sel))/mySize)) * psel)/(xa+xb);
%                     %avgSMnon(imNo,mm)=((B/(sum(sum(non))/mySize)) * pnon)/(xa+xb);
%                     
%                     % dropping the subj
%                     avgSMsel(imNo,mm)=((A/(sum(sum(sel))/mySize)));
%                     avgSMnon(imNo,mm)=((B/(sum(sum(non))/mySize)));
%                 else
                    if peopleChoose % object a is chosen
                        sel = avgSMa;
                        non = avgSMb;
                        psel=xa;
                        pnon=xb;
                    else % object b is chosen
                        sel = avgSMb;
                        non = avgSMa;
                        psel=xb;
                        pnon=xa;
                    end
                    if largest
                        lge = avgSMa;
                        sml = avgSMb;
                    else
                        lge = avgSMb;
                        sml = avgSMa;
                    end
                    
                    %avgSMsel(imNo,mm)=(sel * psel)/(xa+xb);
                    %avgSMnon(imNo,mm)=(non * pnon)/(xa+xb);
                    
                    % dropping the subj
                    avgSMsel(imNo,mm)=sel;
                    avgSMnon(imNo,mm)=non;
                    avgSMbgd(imNo,mm)=avgSMc;
                    avgSMlge(imNo,mm)=lge;
                    avgSMsml(imNo,mm)=sml;
%                 end

%                 if scoreMode == 1
%                     tPR = cell(length(imEachGT),1);
%                     tROC= cell(length(imEachGT),1);
%                     for me = 1:length(imEachGT)
%                         % calc PR + ROC
%                         [tPR{me} tROC{me}] = calcPR(SM,imEachGT{me},comparisonType);
%                     end
%                 elseif scoreMode == 2
%                     tPR = cell(1);
%                     tROC= cell(1);
%                     maxVote = max(max(imAllGT));
%                     tVote = imAllGT == maxVote;
%                     [tPR{1} tROC{1}] = calcPR(SM,tVote,comparisonType);
%                 elseif scoreMode == 3
%                     tPR = cell(1);
%                     tROC= cell(1);
%                     maxVote = max(max(imAllGT));
%                     tVote = (imAllGT > maxVote*thresholdedVotePercentage);
%                     [tPR{1} tROC{1}] = calcPR(SM,tVote,comparisonType);
%                 else
%                     error('Please specify a score mode');
%                 end
                
                clear SM;
%                 
%                 w=vertcat(tPR{:});
%                 a=w(1:2:end,:); %precision
%                 scorePRpr{imNo,mm}=mean(a,1);
%                 b=w(2:2:end,:); %recall
%                 scorePRre{imNo,mm}=mean(b,1);
%                 
%                 w=vertcat(tROC{:});
%                 a=w(1:2:end,:); %tpp
%                 scoreROCt{imNo,mm}=mean(a,1);
%                 b=w(2:2:end,:); %fpp
%                 scoreROCf{imNo,mm}=mean(b,1);
            else
                printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm},'not found')
            end
            end
        end
    end
    
    
    if length(modelUsed) == 1
        if strcmp(modelUsed{1},'IO')
            save(['score/exp1d_' datasetUsed{ds} '_' date '__' modelUsed{1} num2str(IOtype) '_' num2str(comparisonType) '.mat'],'scoreHit','modelUsed','avgSMsel','avgSMnon')
        else
            save(['score/exp1d_' datasetUsed{ds} '_' date '__' modelUsed{1} '_' num2str(comparisonType) '.mat'],'scoreHit','modelUsed','avgSMsel','avgSMnon')
        end
    else
        save(['score/exp1d_' datasetUsed{ds} '_' date '__modelSegmenters_' num2str(comparisonType) '.mat'],'scoreHit','modelUsed','avgSMsel','avgSMnon')
    end
end
% exit

figure
scoreHitPerc=zeros(length(modelUsed),1);
for i=1:length(modelUsed)
    x=vertcat(scoreHit{:,i});
    scoreHitPerc(i)=mean(x);
end
bar(scoreHitPerc)
set(gca,'xtick',1:length(modelUsed),'xticklabel',modelUsed)

figure
scoreAvg=zeros(length(modelUsed),3);
scoreStd=zeros(length(modelUsed),3);
for i=1:length(modelUsed)
    x=avgSMsel(:,i); % selected
    y=avgSMnon(:,i); % non selected
    z=avgSMbgd(:,i); % background
    %v=avgSMlge(:,i); % largest
    %w=avgSMsml(:,i); % smallest
    [a b] = ttest(x,y);
    if a==0
        fprintf('%s sel-non is not significant with score %f\n',modelUsed{i},b);
    else
        fprintf('%s sel-non is significant with score %f\n',modelUsed{i},b);
    end
    [a b] = ttest(x,z);
    if a==0
        fprintf('%s sel-bgd is not significant with score %f\n',modelUsed{i},b);
    else
        fprintf('%s sel-bgd is significant with score %f\n',modelUsed{i},b);
    end
    [a b] = ttest(y,z);
    if a==0
        fprintf('%s non-bgd is not significant with score %f\n',modelUsed{i},b);
    else
        fprintf('%s non-bgd is significant with score %f\n',modelUsed{i},b);
    end
    scoreAvg(i,1)=mean(x);
    scoreAvg(i,2)=mean(y);
    scoreAvg(i,3)=mean(z);
    %scoreAvg(i,4)=mean(v);
    %scoreAvg(i,5)=mean(w);
    scoreStd(i,1)=std(x)/sqrt(length(x));
    scoreStd(i,2)=std(y)/sqrt(length(y));
    scoreStd(i,3)=std(z)/sqrt(length(z));
    %scoreStd(i,4)=std(v)/sqrt(length(v));
    %scoreStd(i,5)=std(w)/sqrt(length(w));
end
%bar(scoreAvg)
barweb(scoreAvg,scoreStd)
set(gca,'xtick',1:length(modelUsed),'xticklabel',modelUsed)
legend('Selected Obj','Non-selected Obj','Background')
%%% WARNING: MEP is actually MOP



figure
scoreAvg=zeros(length(modelUsed),2);
scoreStd=zeros(length(modelUsed),2);
for i=1:length(modelUsed)
    x=hitSMsel(:,i); % selected
%     y=hitSMnon(:,i); % non selected
%     z=hitSMbgd(:,i); % background
    %v=avgSMlge(:,i); % largest
    %w=avgSMsml(:,i); % smallest
%     [a b] = ttest(x,y);
%     if a==0
%         fprintf('%s sel-non is not significant with score %f\n',modelUsed{i},b);
%     else
%         fprintf('%s sel-non is significant with score %f\n',modelUsed{i},b);
%     end
%     [a b] = ttest(x,z);
%     if a==0
%         fprintf('%s sel-bgd is not significant with score %f\n',modelUsed{i},b);
%     else
%         fprintf('%s sel-bgd is significant with score %f\n',modelUsed{i},b);
%     end
    scoreAvg(i,1)=mean(x);
%     scoreAvg(i,2)=mean(y);
%     scoreAvg(i,3)=mean(z);
    %scoreAvg(i,4)=mean(v);
    %scoreAvg(i,5)=mean(w);
    scoreStd(i,1)=std(x)/sqrt(length(x));
%     scoreStd(i,2)=std(y)/sqrt(length(y));
%     scoreStd(i,3)=std(z)/sqrt(length(z));
    %scoreStd(i,4)=std(v)/sqrt(length(v));
    %scoreStd(i,5)=std(w)/sqrt(length(w));
end
%bar(scoreAvg)
barweb(scoreAvg,scoreStd)
set(gca,'xtick',1:length(modelUsed),'xticklabel',modelUsed)
legend('Selected Obj','Non-selected Obj','Background')
% legend('Selected Obj','Non-selected Obj')
title('hit percentage')
kk=mean(rndSel);
% kk=0.5;
hold on
plot([0 length(modelUsed)+1],[kk kk],'--k')
%%% WARNING: MEP is actually MOP


figure
scoreAvg=zeros(length(modelUsed),3);
scoreStd=zeros(length(modelUsed),3);
for i=1:length(modelUsed)
    x=subjSMsel(:,i)./70; % selected
    y=subjSMnon(:,i)./70; % non selected
    z=subjSMbgd(:,i); % background
    [a b] = ttest(x,y);
    if a==0
        fprintf('%s sel-non is not significant with score %f\n',modelUsed{i},b);
    else
        fprintf('%s sel-non is significant with score %f\n',modelUsed{i},b);
    end
    [a b] = ttest(x,z);
    if a==0
        fprintf('%s sel-bgd is not significant with score %f\n',modelUsed{i},b);
    else
        fprintf('%s sel-bgd is significant with score %f\n',modelUsed{i},b);
    end
    scoreAvg(i,1)=mean(x);
    scoreAvg(i,2)=mean(y);
    scoreAvg(i,3)=mean(z);
    scoreStd(i,1)=std(x)/sqrt(length(x));
    scoreStd(i,2)=std(y)/sqrt(length(y));
    scoreStd(i,3)=std(z)/sqrt(length(z));
end
%bar(scoreAvg)
barweb(scoreAvg,scoreStd)
set(gca,'xtick',1:length(modelUsed),'xticklabel',modelUsed)
legend('Selected Obj','Non-selected Obj','Background')
ylim([0 1])
% legend('Selected Obj','Non-selected Obj')
title('saliency value')

% figure
% scoreAvg=zeros(length(modelUsed),2);
% scoreStd=zeros(length(modelUsed),2);
% for i=1:length(modelUsed)
%     x=subjSMsel(:,i);
%     scoreAvg(i,1)=mean(x);
%     scoreStd(i,1)=std(x)/sqrt(length(x));
% end
% %bar(scoreAvg)
% barweb(scoreAvg,scoreStd)
% set(gca,'xtick',1:length(modelUsed),'xticklabel',modelUsed)
% %legend('Selected Obj','Non-selected Obj','Background')
% legend('Avg MaxSalVal selected Obj')
% title('hit percentage')
% %kk=mean(rndSel);
% kk=0.5;
% hold on
% plot([0 length(modelUsed)+1],[kk kk],'--k')
% %%% WARNING: MEP is actually MOP



%%%%%%%%%%%%%%%%%%%%%%%%5
% threshold only the consistent ones
myRatio=max(myObjCount,[],2)/70;
myIdx=myRatio > .75;

figure
scoreAvg=zeros(length(modelUsed),2);
scoreStd=zeros(length(modelUsed),2);
for i=1:length(modelUsed)
    x=hitSMsel(myIdx,i); % selected
    y=hitSMnon(myIdx,i); % non selected
    %z=hitSMbgd(myIdx,i); % background
    
    % ali's sudden idea to make this binary
    x=x>.75;
    y=y>.75;
    %z=z>.75;
    
%     [a b] = ttest(x,y);
%     if a==0
%         fprintf('%s sel-non is not significant with score %f\n',modelUsed{i},b);
%     else
%         fprintf('%s sel-non is significant with score %f\n',modelUsed{i},b);
%     end
%     [a b] = ttest(x,z);
%     if a==0
%         fprintf('%s sel-bgd is not significant with score %f\n',modelUsed{i},b);
%     else
%         fprintf('%s sel-bgd is significant with score %f\n',modelUsed{i},b);
%     end
    scoreAvg(i,1)=mean(x);
    scoreAvg(i,2)=mean(y);
    %scoreAvg(i,3)=mean(z);
    %scoreAvg(i,4)=mean(v);
    %scoreAvg(i,5)=mean(w);
    scoreStd(i,1)=std(x)/sqrt(length(x));
    scoreStd(i,2)=std(y)/sqrt(length(y));
    %scoreStd(i,3)=std(z)/sqrt(length(z));
    %scoreStd(i,4)=std(v)/sqrt(length(v));
    %scoreStd(i,5)=std(w)/sqrt(length(w));
end
bar(scoreAvg)
%barweb(scoreAvg,scoreStd)
set(gca,'xtick',1:length(modelUsed),'xticklabel',modelUsed)
legend('Selected Obj','Non-selected Obj','Background')
% legend('Selected Obj','Non-selected Obj')
title('hit percentage')
%kk=mean(rndSel);
% kk=0.5;
hold on
plot([0 length(modelUsed)+1],[0.1302 0.1302],'--b') % chance selected
plot([0 length(modelUsed)+1],[0.0698 0.0698],'--g') % chance non
plot([0 length(modelUsed)+1],[0.8000 0.8000],'--r') % chance bg
%%% WARNING: MEP is actually MOP

% ali's code for statistic test
%a = round(hitSMbgd*70);
%WW = []; k = 12; Q = a(:,k)'; for i=1:100; WW = [WW ones(1,Q(i)) zeros(1,70-Q(i))]; end; [h p ci zval] = ztest(WW, 0.7694, 0.421); h

% do random statistic test
msel=cell(100,1);
mnon=cell(100,1);
for i=find(myIdx)'%1:100
    for j=1:100
        msel{i}=[msel{i} ones(1,ranDsel(i,j)) zeros(1,70-ranDsel(i,j))];
        mnon{i}=[mnon{i} ones(1,ranDnon(i,j)) zeros(1,70-ranDnon(i,j))];
    end
end
msel=horzcat(msel{:});
mnon=horzcat(mnon{:});

%meanselnon=mean(abs(msel-mnon));
%stdselnon = std(abs(msel-mnon));
meanselnon=mean((msel-mnon));
stdselnon = std((msel-mnon));


%do per model
numModel = length(modelUsed);
a=round(hitSMsel*70);
b=round(hitSMnon*70);
for mi=1:numModel
    nsel=[]; nnon=[];
    %Q=a(:,mi);
    %R=b(:,mi);
    %Q=a(:,mi);
    %R=b(:,mi);

    % ali's sudden idea to make this binary
    Q=a(myIdx,mi)./70;
    R=b(myIdx,mi)./70;
    Q=Q>.75;
    R=R>.75;
    abc=(Q-R);
    %z=z>.75;
    
    %for i=find(myIdx)'%1:100
    %    nsel=[nsel ones(1,Q(i)) zeros(1,70-Q(i))];
    %    nnon=[nnon ones(1,R(i)) zeros(1,70-R(i))];
    %end
    
    %abc=abs(nsel-nnon);
    %abc=(nsel-nnon);
    %[h p ci zval] = ztest(abc, meanselnon, stdselnon);
    [h p ci zval] = ztest(abc, 0, .707);
    
    if h==0
        fprintf('%s sel-non is not significant with score %f\n',modelUsed{mi},p);
    else
        fprintf('%s sel-non is significant with score %f\n',modelUsed{mi},p);
    end
    fprintf('%s mean %f std %f z %f\n',modelUsed{mi},mean(nsel-nnon),std(nsel-nnon),zval);
    %aa = mean(nsel-nnon);
    aa = mean(Q-R);
    %ba = (aa - meanselnon ) / (stdselnon/sqrt(7000));
    ba = (aa - meanselnon ) / (stdselnon/sqrt(59));
    fprintf('%s calc z %f\n',modelUsed{mi},ba);
end


















%%% the original code
%do per model
numModel = length(modelUsed);
a=round(hitSMsel*70);
b=round(hitSMnon*70);
for mi=1:numModel
    nsel=[]; nnon=[];
    %Q=a(:,mi);
    %R=b(:,mi);
    Q=a(:,mi);
    R=b(:,mi);

    
    for i=1:100
       nsel=[nsel ones(1,Q(i)) zeros(1,70-Q(i))];
       nnon=[nnon ones(1,R(i)) zeros(1,70-R(i))];
    end
    
    %abc=abs(nsel-nnon);
    abc=(nsel-nnon);
    %[h p ci zval] = ztest(abc, meanselnon, stdselnon);
    [h p ci zval] = ztest(abc, 0, .707);
    
    if h==0
        fprintf('%s sel-non is not significant with score %f\n',modelUsed{mi},p);
    else
        fprintf('%s sel-non is significant with score %f\n',modelUsed{mi},p);
    end
    fprintf('%s mean %f std %f z %f\n',modelUsed{mi},mean(nsel-nnon),std(nsel-nnon),zval);
    aa = mean(nsel-nnon);
    %aa = mean(Q-R);
    %ba = (aa - meanselnon ) / (stdselnon/sqrt(7000));
    ba = (aa - 0 ) / (.707/sqrt(7000));
    %ba = (aa - meanselnon ) / (stdselnon/sqrt(59));
    fprintf('%s calc z %f\n',modelUsed{mi},ba);
end