% clear all
% close all
% clc

myTic = tic;

generatePath;

% --------------------------------------------
dims = [50 50]; sigma = 10; % gaussian standard deviation in pixels
P = [round(dims(1)/2) round(dims(2)/2)];
[Xm Ym] = meshgrid(-P(2):P(2), -P(1):P(1)); s = sigma ;
gauss = exp(-((( Xm.^2)+( Ym.^2)) ./ (2* s^2)));

myPath = '/lab/raid/models/saliencyDetection/';

% Score mode,
% 1 = average of individual subject's tag(s)
% 2 = highest vote
% 3 = thresholded vote
scoreMode = 2;

% Comparison (Groundtruth vs salmaps) type
% 1 = object tight segmented
% 2 = bounding boxes / edges of the ground truth
% 3 = BW fill
comparisonType = 1;

% IO type:
% 1 = leave one out method
% 2 = one to one
IOtype=1;

alphaRange=0:.02:1;
%alphaRange=0:.1:1;

% param for #3, percentage of max vote
thresholdedVotePercentage = .5;

datasetUsed = { ...
    %     'ASD'
    %     'MSRA'
    %     'SED'
    'SED2'
    %     'SOD'
    %     'ImgSal'
    %     'Bruce'
    };


% Info: 'modelName' % ModelAcronym . Code . Size . Citation . Dataset avail
% * code: M=matlab only, MC=matlab+C, C=C/C++, S=provided
modelUsed = { ...
    % models that we have (have code) = all datasets calculated
    %     'Goferman'   % CA      . M  . max 250 . Gofferman et.al ICVS 2008 . All
    %     'EDS'        % EDS     . MC . orig    . Rosin et.al PRx/2009      . All
    %     'CBsal'      % CBSAL   . MC . orig    . Jiang et.al BMVC 2011     . All
    %     'Materias'   % JL      . M  . orig    . Jian Li BMVC 2011         . All
    %     'salLiu'     % LIU     . MC . max 200 . Liu et al. CVPR 2007      . All
    %     'SVO'        % SVO     . MC . orig    . Chang et al. ICCV 2011    . All
    %     'MSSS'       % MSSS    . M  . orig    . Achanta ICIP 2008         . All
    %     'AC'         % AC      . M  . orig    . Achanta ICVS 2008         . All
    %     'FTS'        % FT      . M  . orig    . Achanta CVPR 2009         . All
    %     'MichalGazit'% GAZIT   . M  . orig    . Gazit et.al ECCV2010      . All
    % some precomputed models
    %%%%%%'CA' % ASD only- This is Gofferman, Just keep this.
    %%%%%%'RCC' % Thresholded version of RC, ignore this
    %     'MZ'         % MZ      . S  . orig    . Ma and Zhang 2003         . ASD only
    %     'Valenti'    % VALENTI . S  . orig    . Valenti et.al ICCV 2009   . ASD and MSRA only
    %     'DAKlein'    % BITS    . S  . orig    . Klein & Frintrop ICCV 2011. All
    %     'PMehrani'   % MV      . S  . orig    . Mehrani et.al BMVC 2010   . ASD and SED1 only
    %     'LiuICIP'    % LIU-ICIP. S  . orig    . Liu et.al ICIP 2010       . ASD only
    %     'LiuIETIP'   % LIU-TiP . S  . orig    . Liu et.al TiP 2011        . ASD only
    %     'MengW'      % MENG    . S  . orig    .                           . ASD only
    %%precomputed on windoze machine
    %%%%% 'FT' % = FTS, ignore this model
    %%%%% 'SR' % This is the same as HouCVPR
    %     'HC'         % HC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    %     'LC'         % LC      . C  . orig    . Zhai and Shah ACM-M 2006  . All
    %     'RC'         % RC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    %partial result, maybe biased due to train&test on same dataset but different files
    %     'JiaLiSal'   % JIA-LI  . S  . 1/16    . Jia Li et.al IJCV 2010    . Partial ASD and MSRA
    %
    %
    %     % Saliency modeling models (for fixation prediction)
    'SAIM'       % AIM     . M  . 1/2     .
    'SHouCVPR'   % SR      . M  . 64x64   . SRM % SKIP This as it is the same as SR
    'SHouNIPS'   % ICL     . M  . orig    .
    'SSUN'       % SUN     . M  . 1/2 - N .
    'SAWS'       % AWS     . M  . 1/2     .
    'SPQFT'      % PQFT    . M  . 400x400 .
    'SGBVS'      % GBVS    . M  . orig    .have problem when the image is under 128x128, same as GB
    'SITTI'      % ITTI    . C  . 1/16    . Itti model - same as IT
    'SITTI98'    % ITTI98  . C  . 1/16    . Maxnorm
    'SSEO'       % SDSR    . M  . orig    .
    %     'SJudd'      % Judd    . M  . orig    .
    %
    %
    %     % MAP map (Mean Annotation Map)
    %     'MEP' % Mean Annotation Map . 500x500
    %     'IO'  % >>> Inter observer  . orig
    'objSize'
    %
    %     % segmenters, not salient object detections
    %     'SRS1' % SRS . Christian's model, SRS# where # is the #-th salient point
    %     %     'SRS2'
    %     %     'SRS3'
    %     %     'SRS4'
    %     %     'SRS5'
    %     'Mishra' % MISHRA .
    %
    %     % some combination maps, 3 best
    %     'CMAPmlin' % mult linear
    %     'CMAPplin' % plus linear
    %     'CMAPplog' % plus log(x/2)
    %     'CMAPpexp' % plus exp(x^3)
    % %     % 2 best
    % %     'CMAP2mlin' % mult linear
    % %     'CMAP2plin' % plus linear
    % %     'CMAP2plog' % plus log(x/2)
    % %     'CMAP2pexp' % plus exp(x^3)
    
    };

colorCode={'m','c','r','g','b','k'};
lineCode ={'-','--',':','-.'};
theCodes=cell(length(colorCode)*length(lineCode),1);
for i=1:length(colorCode)
    for j=1:length(lineCode)
        theCodes{(i-1)*length(lineCode)+j} = [colorCode{i} lineCode{j}];
    end
end

% loop dataset
for ds=1:length(datasetUsed)
    eval(['imName = ' datasetUsed{ds} 'getList;'])
    myHitAlpha = cell(length(alphaRange),1);
    
    for alphaNo=1:length(alphaRange)
        alphaTune=alphaRange(alphaNo);
        scoreHit = cell(length(imName),length(modelUsed));
        avgSMscore = cell(length(imName),length(modelUsed),5);
        
        avgSMsel = zeros(length(imName),length(modelUsed));
        avgSMnon = zeros(length(imName),length(modelUsed));
        avgSMbgd = zeros(length(imName),length(modelUsed));
        avgSMlge = zeros(length(imName),length(modelUsed));
        avgSMsml = zeros(length(imName),length(modelUsed));
        
        hitSMsel = zeros(length(imName),length(modelUsed));
        hitSMnon = zeros(length(imName),length(modelUsed));
        hitSMbgd = zeros(length(imName),length(modelUsed));
        
        rndSel = zeros(length(imName),1);
        rndNon = zeros(length(imName),1);
        
        subjSMsel = zeros(length(imName),length(modelUsed));
        subjSMnon = zeros(length(imName),length(modelUsed));
        subjSMbgd = zeros(length(imName),length(modelUsed));
        
        %load('/lab/raid/models/saliencyDetection/salJudgeData/exp1Result.mat')
        load('/lab/raid/models/saliencyDetection/salJudgeData/exp1_v2.mat')
        
        % loop image
        for imNo = 1:length(imName)
            for mm=1:length(modelUsed)
                if strcmp(modelUsed{mm},'MEP')
                    theName = [myPath 'output/MEP_' datasetUsed{ds} '.mat'];
                else
                    theName = [myPath 'output/' modelUsed{mm} '/' datasetUsed{ds} '/' rmExt(imName{imNo}) '.mat'];
                end
                if exist(theName,'file') || ...
                        strcmp(modelUsed{mm},'objSize')
                    
                    %%% Loading SM %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    if strcmp(modelUsed{mm},'objSize')
                        a=mySegObj{imNo,1};
                        b=mySegObj{imNo,2};
                        if mean2(a) > mean2(b)
                            SM = a;
                        else
                            SM = b;
                        end
                    else
                        load(theName);
                    end
                    printLog(myTic,'dataset',datasetUsed{ds},'alpha',num2str(alphaTune,'%.3f'),'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
                    
                    SM = double(SM);
                    if size(SM,3)>1
                        SM = rgb2gray(SM);
                    end
                    SM = imresize(SM,size(mySegObj{imNo,1}),'nearest');
                    SM = SM - min(min(SM));
                    SM = SM ./ max(max(SM));
                    
                    % Biggest Object
                    a=mySegObj{imNo,1};
                    b=mySegObj{imNo,2};
                    if mean2(a) > mean2(b)
                        BSM = a;
                    else
                        BSM = b;
                    end
                    
                    % doing map combination
                    SM = (alphaTune.*BSM) + ((1-alphaTune) .* SM);
                    SM = SM - min(min(SM));
                    SM = SM ./ max(max(SM));
                    
                    %if strcmp(modelUsed{mm},'SAWS')
                    %    imshow(SM,[]);
                    %    pause(.3);
                    %end
                    
                    %%% The score system %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    za=mySegObj{imNo,1};
                    zb=mySegObj{imNo,2};
                    
                    % try to expand the segmentation
                    %se = strel('square',9);
                    %za = imdilate(za,se);
                    %zb = imdilate(zb,se);
                    
                    xa=myObjCount(imNo,1);
                    xb=myObjCount(imNo,2);
                    peopleChoose = xa > xb;
                    largest = mean2(za) > mean2(zb);
                    mySize = size(SM);
                    mySize = mySize(1)*mySize(2);
                    SMa = SM .* za;
                    SMb = SM .* zb;
                    SMc = SM .* ~(za | zb);
                    % average
                    %avgSMa = sum(sum(SMa)) ./ sum(sum(za));% ./(sum(sum(za))/mySize);
                    %avgSMb = sum(sum(SMb)) ./ sum(sum(zb));%./(sum(sum(zb))/mySize);
                    % maximum
                    avgSMa = max(max(SMa));
                    avgSMb = max(max(SMb));
                    avgSMc = max(max(SMc));
                    % new
                    %avgSMa = max(max(SMa))./(sum(sum(za))/mySize);
                    %avgSMb = max(max(SMb))./(sum(sum(zb))/mySize);
                    
                    subjSMsel(imNo,mm)=avgSMa*xa + avgSMb*xb;
                    subjSMnon(imNo,mm)=avgSMa*xb + avgSMb*xa;
                    subjSMbgd(imNo,mm)=avgSMc;
                    
                    SMchoose = avgSMa > avgSMb;
                    if peopleChoose == SMchoose
                        scoreHit{imNo,mm} = 1;
                    else
                        scoreHit{imNo,mm} = 0;
                    end
                    
                    avgSMscore{imNo,mm,1}=avgSMa;
                    avgSMscore{imNo,mm,2}=avgSMb;
                    avgSMscore{imNo,mm,3}=avgSMc;
                    
                    %%% Accuracy hit %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    mySMbin = SM .* (za | zb); % binary decision
                    mySMbin = mySMbin ./ max(max(mySMbin));
                    
                    %%% adding SM OR Largest
                    a=mySegObj{imNo,1};
                    b=mySegObj{imNo,2};
                    if mean2(a) > mean2(b)
                        SMlge = a;
                    else
                        SMlge = b;
                    end
                    
                    %mySMhit = (mySMbin == 1) | SMlge; %%% <---------------------
                    mySMhit = (mySMbin == 1); %%%%%% Without Largest
                    %mySMhit = (SM == 1); %%%%%% Raw Saliency
                    %if mean2(~(za | zb) & mySMhit) > 0
                    %    hitSMbgd(imNo,mm)=1;
                    %                 end
                    %else
                    if mean2((za | zb) & mySMhit) > 0
                        if peopleChoose % object a is chosen(higher)
                            if mean2(za & mySMhit) > 0 % hit on A
                                %hitSMsel(imNo,mm)=1;
                                hitSMsel(imNo,mm)=xa/70;
                                hitSMnon(imNo,mm)=xb/70;
                            else % miss
                                hitSMsel(imNo,mm)=xb/70;
                                hitSMnon(imNo,mm)=xa/70;
                            end
                            %                     if mean2(zb & mySMhit) == 0
                            %                         %hitSMnon(imNo,mm)=1;
                            %                         hitSMnon(imNo,mm)=xb/70;
                            %                     else
                            %                         hitSMnon(imNo,mm)=xa/70;
                            %                     end
                            rndSel(imNo) = sum(sum(za)) / (size(za,1)*size(za,2));
                            rndNon(imNo) = sum(sum(zb)) / (size(zb,1)*size(zb,2));
                        else % object b is chosen (higher)
                            if mean2(zb & mySMhit) > 0
                                %hitSMsel(imNo,mm)=1;
                                hitSMsel(imNo,mm)=xb/70;
                                hitSMnon(imNo,mm)=xa/70;
                            else
                                hitSMsel(imNo,mm)=xa/70;
                                hitSMnon(imNo,mm)=xb/70;
                            end
                            %                     if mean2(za & mySMhit) == 0
                            %                         %hitSMnon(imNo,mm)=1;
                            %                         hitSMnon(imNo,mm)=xa/70;
                            %                     else
                            %                         hitSMnon(imNo,mm)=xb/70;
                            %                     end
                            rndSel(imNo) = sum(sum(zb)) / (size(zb,1)*size(zb,2));
                            rndNon(imNo) = sum(sum(za)) / (size(za,1)*size(za,2));
                        end
                    else
                        hitSMbgd(imNo,mm)=1;
                    end
                    
                    
                    if peopleChoose % object a is chosen
                        sel = avgSMa;
                        non = avgSMb;
                        psel=xa;
                        pnon=xb;
                    else % object b is chosen
                        sel = avgSMb;
                        non = avgSMa;
                        psel=xb;
                        pnon=xa;
                    end
                    if largest
                        lge = avgSMa;
                        sml = avgSMb;
                    else
                        lge = avgSMb;
                        sml = avgSMa;
                    end
                    
                    %avgSMsel(imNo,mm)=(sel * psel)/(xa+xb);
                    %avgSMnon(imNo,mm)=(non * pnon)/(xa+xb);
                    
                    % dropping the subj
                    avgSMsel(imNo,mm)=sel;
                    avgSMnon(imNo,mm)=non;
                    avgSMbgd(imNo,mm)=avgSMc;
                    avgSMlge(imNo,mm)=lge;
                    avgSMsml(imNo,mm)=sml;

                    clear SM;
                else
                    printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm},'not found')
                end
            end
        end
        
        myHitAlpha{alphaNo}=hitSMsel;
    end
end


% figure
% scoreAvg=zeros(length(modelUsed),2);
% scoreStd=zeros(length(modelUsed),2);
% for i=1:length(modelUsed)
%     x=hitSMsel(:,i); % selected
%     scoreAvg(i,1)=mean(x);
%     scoreStd(i,1)=std(x)/sqrt(length(x));
% end
% %bar(scoreAvg)
% barweb(scoreAvg,scoreStd)
% set(gca,'xtick',1:length(modelUsed),'xticklabel',modelUsed)
% legend('Selected Obj','Non-selected Obj','Background')
% % legend('Selected Obj','Non-selected Obj')
% title('hit percentage')
% kk=mean(rndSel);
% % kk=0.5;
% hold on
% plot([0 length(modelUsed)+1],[kk kk],'--k')
% %%% WARNING: MEP is actually MOP
figure
myScoreAll=cat(3,myHitAlpha{:});
myScoreAll=permute(myScoreAll,[3 2 1]);
myScoreAvg=mean(myScoreAll,3);
myScoreStd=std(myScoreAll,[],3)./sqrt(size(myScoreAll,3));
%errorbar(myScoreAvg,myScoreStd)
%barweb(myScoreAvg,myScoreStd)
plot(myScoreAvg)
legend(modelUsed)
set(gca,'xtick',1:length(alphaRange),'xticklabel',alphaRange);
ylim([.5 .8])
xlim([0.5 length(alphaRange)+.5])

% save(['score/exp1graph_' datasetUsed{ds} '_' date '__model.mat'],'myScoreAll','modelUsed','alphaRange')