clear all
close all
clc

myTic = tic;

generatePath;

myPath = '/lab/raid/models/saliencyDetection/';

% Score mode,
% 1 = average of individual subject's tag(s)
% 2 = highest vote
% 3 = thresholded vote
scoreMode = 1;

% Comparison (Groundtruth vs salmaps) type
% 1 = object tight segmented
% 2 = bounding boxes / edges of the ground truth
% 3 = BW fill
comparisonType = 1;

% param for #3, percentage of max vote
thresholdedVotePercentage = .5;

gaussSize=[ 5 15];

datasetUsed = { ...
%     'ASD'
%     'MSRA'
    'SED'
    'SED2'
    'SOD'
    };


modelUsed = { ...
    % models that we have (have code) = all datasets calculated
    'Goferman'   % CA      . M  . max 250 . Gofferman et.al ICVS 2008 . All
    'EDS'        % EDS     . MC . orig    . Rosin et.al PRx/2009      . All
    'CBsal'      % CBSAL   . MC . orig    . Jiang et.al BMVC 2011     . All
    'Materias'   % JL      . M  . orig    . Jian Li BMVC 2011         . All
    'salLiu'     % LIU     . MC . max 200 . Liu et al. CVPR 2007      . All
    'SVO'        % SVO     . MC . orig    . Chang et al. ICCV 2011    . All
    'MSSS'       % MSSS    . M  . orig    . Achanta ICIP 2008         . All
    'AC'         % AC      . M  . orig    . Achanta ICVS 2008         . All
    'FTS'        % FT      . M  . orig    . Achanta CVPR 2009         . All
    'MichalGazit'% GAZIT   . M  . orig    . Gazit et.al ECCV2010      . All
    % some precomputed models
    %%%%%%'CA' % ASD only- This is Gofferman, Just keep this.
    %%%%%%'RCC' % Thresholded version of RC, ignore this
    'MZ'         % MZ      . S  . orig    . Ma and Zhang 2003         . ASD only
    'Valenti'    % VALENTI . S  . orig    . Valenti et.al ICCV 2009   . ASD and MSRA only
    'DAKlein'    % BITS    . S  . orig    . Klein & Frintrop ICCV 2011. All
    'PMehrani'   % MV      . S  . orig    . Mehrani et.al BMVC 2010   . ASD and SED1 only
    'LiuICIP'    % LIU-ICIP. S  . orig    . Liu et.al ICIP 2010       . ASD only
    'LiuIETIP'   % LIU-TiP . S  . orig    . Liu et.al TiP 2011        . ASD only
    'MengW'      % MENG    . S  . orig    .                           . ASD only
    %%precomputed on windoze machine
    %%%%% 'FT' % = FTS, ignore this model
    %%%%% 'SR' % This is the same as HouCVPR
    'HC'         % HC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    'LC'         % LC      . C  . orig    . Zhai and Shah ACM-M 2006  . All
    'RC'         % RC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    %partial result, maybe biased due to train&test on same dataset but different files
    'JiaLiSal'   % JIA-LI  . S  . 1/16    . Jia Li et.al IJCV 2010    . Partial ASD and MSRA
    
    
    % Saliency modeling models (for fixation prediction)
    'SAIM'       % AIM     . M  . 1/2     .
    'SHouCVPR'   % SR      . M  . 64x64   . SRM % SKIP This as it is the same as SR
    'SHouNIPS'   % ICL     . M  . orig    .
    'SSUN'       % SUN     . M  . 1/2 - N .
    'SAWS'       % AWS     . M  . 1/2     .
    'SPQFT'      % PQFT    . M  . 400x400 .
    'SGBVS'      % GBVS    . M  . orig    .have problem when the image is under 128x128, same as GB
    'SITTI'      % ITTI    . C  . 1/16    . Itti model - same as IT
    'SITTI98'    % ITTI98  . C  . 1/16    . Maxnorm
    'SSEO'       % SDSR    . M  . orig    .
    'SJudd'      % Judd    . M  . orig    .
    
    
    % MAP map (Mean Annotation Map)
    'MEP' % Mean Annotation Map . 500x500
%     'IO'  % >>> Inter observer  . orig
    
    % segmenters, not salient object detections
    'SRS1' % SRS . Christian's model, SRS# where # is the #-th salient point
    %     'SRS2'
    %     'SRS3'
    %     'SRS4'
    %     'SRS5'
    'Mishra' % MISHRA .
    
    % some combination maps, 3 best
    'CMAPmlin' % mult linear
    'CMAPplin' % plus linear
    'CMAPplog' % plus log(x/2)
    'CMAPpexp' % plus exp(x^3)
%     % 2 best
%     'CMAP2mlin' % mult linear
%     'CMAP2plin' % plus linear
%     'CMAP2plog' % plus log(x/2)
%     'CMAP2pexp' % plus exp(x^3)

    };


% loop dataset
for ds=1:length(datasetUsed)
    eval(['imName = ' datasetUsed{ds} 'getList;'])
    
    scorePRpr = cell(length(imName),length(modelUsed),length(gaussSize)); %precision
    scorePRre = cell(length(imName),length(modelUsed),length(gaussSize)); %recall
    scoreROCt = cell(length(imName),length(modelUsed),length(gaussSize)); %tpp?
    scoreROCf = cell(length(imName),length(modelUsed),length(gaussSize)); %fpp?
    % loop image
    for imNo = 1:length(imName)
        eval(['[imBox imEachGT imAllGT] = ' datasetUsed{ds} 'getBox(imName{imNo});'])
        
        % loop models
        for mm=1:length(modelUsed)
            printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
            
                
            if strcmp(modelUsed{mm},'MEP')
                theName = [myPath 'output/MEP_' datasetUsed{ds} '.mat'];
            else
                theName = [myPath 'output/' modelUsed{mm} '/' datasetUsed{ds} '/' rmExt(imName{imNo}) '.mat'];
            end

            if exist(theName,'file')
                for gg=1:length(gaussSize)
                    
                    % --------------------------------------------
                    dims = [50 50]; sigma = gaussSize(gg); % gaussian standard deviation in pixels
                    P = [round(dims(1)/2) round(dims(2)/2)];
                    [Xm Ym] = meshgrid(-P(2):P(2), -P(1):P(1)); s = sigma ;
                    gauss = exp(-((( Xm.^2)+( Ym.^2)) ./ (2* s^2)));
                    
                    load(theName);
                    
                    if size(SM,3) == 3
                        SM = rgb2gray(SM);
                    end
                    SM = imresize(SM,size(imEachGT{1}),'nearest');
                    SM = conv2(double(SM),gauss,'same');
                    
                    if scoreMode == 1
                        tPR = cell(length(imEachGT),1);
                        tROC= cell(length(imEachGT),1);
                        for me = 1:length(imEachGT)
                            % calc PR + ROC
                            [tPR{me} tROC{me}] = calcPR(SM,imEachGT{me},comparisonType);
                        end
                    elseif scoreMode == 2
                        tPR = cell(1);
                        tROC= cell(1);
                        maxVote = max(max(imAllGT));
                        tVote = imAllGT == maxVote;
                        [tPR{1} tROC{1}] = calcPR(SM,tVote,comparisonType);
                    elseif scoreMode == 3
                        tPR = cell(1);
                        tROC= cell(1);
                        maxVote = max(max(imAllGT));
                        tVote = (imAllGT > maxVote*thresholdedVotePercentage);
                        [tPR{1} tROC{1}] = calcPR(SM,tVote,comparisonType);
                    else
                        error('Please specify a score mode');
                    end
                    
                    clear SM;
                    
                    w=vertcat(tPR{:});
                    a=w(1:2:end,:); %precision
                    scorePRpr{imNo,mm,gg}=mean(a,1);
                    b=w(2:2:end,:); %recall
                    scorePRre{imNo,mm,gg}=mean(b,1);
                    
                    w=vertcat(tROC{:});
                    a=w(1:2:end,:); %tpp
                    scoreROCt{imNo,mm,gg}=mean(a,1);
                    b=w(2:2:end,:); %fpp
                    scoreROCf{imNo,mm,gg}=mean(b,1);
                    
                end
            end
        end
    end
    
    
    
    save(['score/' datasetUsed{ds} '_' date '__gauss.mat'],'scorePRpr','scorePRre','scoreROCt','scoreROCf','modelUsed')
end