clear all
close all
clc

myTic = tic;

generatePath;

% --------------------------------------------
dims = [50 50]; sigma = 10; % gaussian standard deviation in pixels
P = [round(dims(1)/2) round(dims(2)/2)];
[Xm Ym] = meshgrid(-P(2):P(2), -P(1):P(1)); s = sigma ;
gauss = exp(-((( Xm.^2)+( Ym.^2)) ./ (2* s^2)));

myPath = '/lab/raid/models/saliencyDetection/';

% Score mode,
% 1 = average of individual subject's tag(s)
% 2 = highest vote
% 3 = thresholded vote
scoreMode = 1;

% Comparison (Groundtruth vs salmaps) type
% 1 = object tight segmented
% 2 = bounding boxes / edges of the ground truth
% 3 = BW fill
comparisonType = 3;

% param for #3, percentage of max vote
thresholdedVotePercentage = .5;

% param for percentage of MAP
mapPercentage = 1.0;

datasetUsed = { ...
%     'ASD'
%     'MSRA'
    'SED'
    'SED2'
    'SOD'
    };


% Info: 'modelName' % ModelAcronym . Code . Size . Citation . Dataset avail
% * code: M=matlab only, MC=matlab+C, C=C/C++, S=provided
modelUsed = { ...
    % models that we have (have code) = all datasets calculated
    'Goferman'   % CA      . M  . max 250 . Gofferman et.al ICVS 2008 . All
    'EDS'        % EDS     . MC . orig    . Rosin et.al PRx/2009      . All
    'CBsal'      % CBSAL   . MC . orig    . Jiang et.al BMVC 2011     . All
    'Materias'   % JL      . M  . orig    . Jian Li BMVC 2011         . All
    'salLiu'     % LIU     . MC . max 200 . Liu et al. CVPR 2007      . All
    'SVO'        % SVO     . MC . orig    . Chang et al. ICCV 2011    . All
    'MSSS'       % MSSS    . M  . orig    . Achanta ICIP 2008         . All
    'AC'         % AC      . M  . orig    . Achanta ICVS 2008         . All
    'FTS'        % FT      . M  . orig    . Achanta CVPR 2009         . All
    'MichalGazit'% GAZIT   . M  . orig    . Gazit et.al ECCV2010      . All
    % some precomputed models
    %%%%%%'CA' % ASD only- This is Gofferman, Just keep this.
    %%%%%%'RCC' % Thresholded version of RC, ignore this
    'MZ'         % MZ      . S  . orig    . Ma and Zhang 2003         . ASD only
    'Valenti'    % VALENTI . S  . orig    . Valenti et.al ICCV 2009   . ASD and MSRA only
    'DAKlein'    % BITS    . S  . orig    . Klein & Frintrop ICCV 2011. All
    'PMehrani'   % MV      . S  . orig    . Mehrani et.al BMVC 2010   . ASD and SED1 only
    'LiuICIP'    % LIU-ICIP. S  . orig    . Liu et.al ICIP 2010       . ASD only
    'LiuIETIP'   % LIU-TiP . S  . orig    . Liu et.al TiP 2011        . ASD only
    'MengW'      % MENG    . S  . orig    .                           . ASD only
    %%precomputed on windoze machine
    %%%%% 'FT' % = FTS, ignore this model
    %%%%% 'SR' % This is the same as HouCVPR
    'HC'         % HC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    'LC'         % LC      . C  . orig    . Zhai and Shah ACM-M 2006  . All
    'RC'         % RC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    %partial result, maybe biased due to train&test on same dataset but different files
    'JiaLiSal'   % JIA-LI  . S  . 1/16    . Jia Li et.al IJCV 2010    . Partial ASD and MSRA
    
    
    % Saliency modeling models (for fixation prediction)
    'SAIM'       % AIM     . M  . 1/2     .
    'SHouCVPR'   % SR      . M  . 64x64   . SRM % SKIP This as it is the same as SR
    'SHouNIPS'   % ICL     . M  . orig    .
    'SSUN'       % SUN     . M  . 1/2 - N .
    'SAWS'       % AWS     . M  . 1/2     .
    'SPQFT'      % PQFT    . M  . 400x400 .
    'SGBVS'      % GBVS    . M  . orig    .have problem when the image is under 128x128, same as GB
    'SITTI'      % ITTI    . C  . 1/16    . Itti model - same as IT
    'SITTI98'    % ITTI98  . C  . 1/16    . Maxnorm
    'SSEO'       % SDSR    . M  . orig    .
    'SJudd'      % Judd    . M  . orig    .
    
    
    % MAP map (Mean Annotation Map)
    'MEP' % Mean Annotation Map . 500x500
    'IO'  % >>> Inter observer  . orig
    
    % segmenters, not salient object detections
    'SRS1' % SRS . Christian's model, SRS# where # is the #-th salient point
        %'SRS2'
        %'SRS3'
        %'SRS4'
        %'SRS5'
    'Mishra' % MISHRA .
    
    % some combination maps, 3 best
    'CMAPmlin' % mult linear
    'CMAPplin' % plus linear
    'CMAPplog' % plus log(x/2)
    'CMAPpexp' % plus exp(x^3)
    % 2 best
    'CMAP2mlin' % mult linear
    'CMAP2plin' % plus linear
    'CMAP2plog' % plus log(x/2)
    'CMAP2pexp' % plus exp(x^3)

    };

colorCode={'m','c','r','g','b','k'};
lineCode ={'-','--',':','-.'};
theCodes=cell(length(colorCode)*length(lineCode),1);
for i=1:length(colorCode)
    for j=1:length(lineCode)
        theCodes{(i-1)*length(lineCode)+j} = [colorCode{i} lineCode{j}];
    end
end

% loop dataset
for ds=1:length(datasetUsed)
    eval(['imName = ' datasetUsed{ds} 'getList;'])
    
    scorePRpr = cell(length(imName),length(modelUsed)); %precision
    scorePRre = cell(length(imName),length(modelUsed)); %recall
    scoreROCt = cell(length(imName),length(modelUsed)); %tpp?
    scoreROCf = cell(length(imName),length(modelUsed)); %fpp?
    % loop image
    
    theName = [myPath 'output/MEP_' datasetUsed{ds} '.mat'];
    load(theName);
    myMAP=SM;
    myMAP = myMAP - min(min(myMAP));
    myMAP = myMAP ./ max(max(myMAP));
    myMAP = myMAP .* mapPercentage;
    for imNo = 1:length(imName)
        eval(['[imBox imEachGT imAllGT] = ' datasetUsed{ds} 'getBox(imName{imNo});'])
        
        % loop models
        for mm=1:length(modelUsed)
            %printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
            if strcmp(modelUsed{mm},'IO') % Inter observer model
                if strcmp(datasetUsed{ds},'ASD'); continue; end % ASD doesnt have IO
                printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
                
                tPR = cell(length(imEachGT),length(imEachGT));
                tROC= cell(length(imEachGT),length(imEachGT));

                % inter observer: take n-th, compare with 1 ~ n-1 stuffs
                for me = 1:length(imEachGT)
                    SM = double(imEachGT{me});
                    SM = conv2(SM,gauss,'same');
                    for mu = 1:length(imEachGT)
                        if me == mu; continue; end
                        % calc PR + ROC
%                         subplot(1,2,1); imshow(SM,[]);
%                         subplot(1,2,2); imshow(imEachGT{mu},[]);
%                         pause(0.3);
                        [tPR{me,mu} tROC{me,mu}] = calcPR(SM,imEachGT{mu},comparisonType);
                    end
                end
                w=vertcat(tPR{:});
                a=w(1:2:end,:); %precision
                scorePRpr{imNo,mm}=mean(a,1);
                b=w(2:2:end,:); %recall
                scorePRre{imNo,mm}=mean(b,1);
                
                w=vertcat(tROC{:});
                a=w(1:2:end,:); %tpp
                scoreROCt{imNo,mm}=mean(a,1);
                b=w(2:2:end,:); %fpp
                scoreROCf{imNo,mm}=mean(b,1);
            else
            if strcmp(modelUsed{mm},'MEP')
                theName = [myPath 'output/MEP_' datasetUsed{ds} '.mat'];
            else
                theName = [myPath 'output/' modelUsed{mm} '/' datasetUsed{ds} '/' rmExt(imName{imNo}) '.mat'];
            end
            if exist(theName,'file')
                load(theName);
                %printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm}, ...
                %    'size =',size(SM,2),'x',size(SM,1), 'multiplier =',size(SM,2)/size(imAllGT,2),'x',size(SM,1)/size(imAllGT,1))
                printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm});
                
                if scoreMode == 1
                    tPR = cell(length(imEachGT),1);
                    tROC= cell(length(imEachGT),1);
                    for me = 1:length(imEachGT)
                        % calc PR + ROC
                        if strcmp(modelUsed{mm},'MEP')
                            [tPR{me} tROC{me}] = calcPR(SM,imEachGT{me},comparisonType);
                        else
                            [tPR{me} tROC{me}] = calcPRmap(SM,imEachGT{me},myMAP,comparisonType);
                        end
                    end
                elseif scoreMode == 2
                    tPR = cell(1);
                    tROC= cell(1);
                    maxVote = max(max(imAllGT));
                    tVote = imAllGT == maxVote;
                    if strcmp(modelUsed{mm},'MEP')
                        [tPR{1} tROC{1}] = calcPR(SM,tVote,comparisonType);
                    else
                        [tPR{1} tROC{1}] = calcPRmap(SM,tVote,myMAP,comparisonType);
                    end
                elseif scoreMode == 3
                    tPR = cell(1);
                    tROC= cell(1);
                    maxVote = max(max(imAllGT));
                    tVote = (imAllGT > maxVote*thresholdedVotePercentage);
                    if strcmp(modelUsed{mm},'MEP')
                        [tPR{1} tROC{1}] = calcPR(SM,tVote,comparisonType);
                    else
                    [tPR{1} tROC{1}] = calcPRmap(SM,tVote,myMAP,comparisonType);
                    end
                else
                    error('Please specify a score mode');
                end
                
                clear SM;
                
                w=vertcat(tPR{:});
                a=w(1:2:end,:); %precision
                scorePRpr{imNo,mm}=mean(a,1);
                b=w(2:2:end,:); %recall
                scorePRre{imNo,mm}=mean(b,1);
                
                w=vertcat(tROC{:});
                a=w(1:2:end,:); %tpp
                scoreROCt{imNo,mm}=mean(a,1);
                b=w(2:2:end,:); %fpp
                scoreROCf{imNo,mm}=mean(b,1);
            else
                printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm},'not found')
            end
            end
        end
    end
%     
%     Fmeasure = zeros(length(modelUsed),1);
%     beta=.3; % whats this?
%     figure; hold on
%     for mm=1:length(modelUsed)
%         pr = vertcat(scorePRpr{:,mm});
%         re = vertcat(scorePRre{:,mm});
%         % remove all NaNs
%         pr(isnan(pr)) = 0;
%         re(isnan(re)) = 0;
%         %plot(mean(re),mean(pr),'color',rand(1,3));
%         plot(mean(re),mean(pr),theCodes{mod(mm-1,length(theCodes))+1},'linewidth',ceil(mm./length(theCodes)) );
%         Fmeasure(mm) = ((1+ beta)*mean2(pr) * mean2(re)) / (beta*mean2(re) + mean2(pr));
%     end
%     hold off
%     ylim([0 1])
%     xlim([0 1])
%     xlabel('recall')
%     ylabel('precision')
%     legend(modelUsed);
%     
%     figure;
%     bar(Fmeasure)
%     set(gca, 'XTick',  1:length(modelUsed))
%     set(gca, 'XTickLabel', modelUsed)
%     
%     aucs = zeros(length(modelUsed),1);
%     figure; hold on
%     for mm=1:length(modelUsed)
%         tpp = vertcat(scoreROCt{:,mm});
%         fpp = vertcat(scoreROCf{:,mm});
%         % remove all NaNs
%         tpp(isnan(tpp)) = 0;
%         fpp(isnan(fpp)) = 0;
%         %plot(mean(fpp),mean(tpp),'color',rand(1,3));
%         plot(mean(fpp),mean(tpp),theCodes{mod(mm-1,length(theCodes))+1},'linewidth',ceil(mm./length(theCodes)) );
%         aucs(mm) = -trapz(mean(fpp),mean(tpp));
%     end
%     plot(0:.1:1,0:.1:1,'k', 'linewidth',2)
%     hold off
%     ylim([0 1])
%     xlim([0 1])
%     ylabel('true positive')
%     xlabel('false positive')
%     legend(modelUsed);
%     
%     figure;
%     bar(aucs)
%     set(gca,'ylim',[.5 1])
%     set(gca,'xlim',[.5 length(modelUsed)+.5])
%     set(gca, 'XTick',  1:length(modelUsed))
%     set(gca, 'XTickLabel', modelUsed)
    if length(modelUsed) == 1
        save(['score/' datasetUsed{ds} '_' date '__' modelUsed{1} '_' num2str(comparisonType) '_MAP' num2str(mapPercentage,'%1.1f') '.mat'],'scorePRpr','scorePRre','scoreROCt','scoreROCf','modelUsed')
    else
        save(['score/' datasetUsed{ds} '_' date '__modelSegmenters_' num2str(comparisonType) '_MAP' num2str(mapPercentage,'%1.1f') '.mat'],'scorePRpr','scorePRre','scoreROCt','scoreROCf','modelUsed')
        %save(['score/' datasetUsed{ds} '_' date '__modelSal_' num2str(comparisonType) '.mat'],'scorePRpr','scorePRre','scoreROCt','scoreROCf','modelUsed')
    end
end
exit