clear all
close all
clc

myTic = tic;

generatePath;

myPath = '/lab/raid/models/saliencyDetection/';



% param for #3, percentage of max vote
thresholdedVotePercentage = .5;

datasetUsed = { ...
%     'ASD'
    'MSRA'
%     'SED'
%     'SED2'
%     'SOD'
    };


% Info: 'modelName' % ModelAcronym . Code . Size . Citation . Dataset avail
% * code: M=matlab only, MC=matlab+C, C=C/C++, S=provided
modelUsed = { ...
    % models that we have (have code) = all datasets calculated
    'Goferman'   % CA      . M  . max 250 . Gofferman et.al ICVS 2008 . All
    'EDS'        % EDS     . MC . orig    . Rosin et.al PRx/2009      . All
    'CBsal'      % CBSAL   . MC . orig    . Jiang et.al BMVC 2011     . All
    'Materias'   % JL      . M  . orig    . Jian Li BMVC 2011         . All
    'salLiu'     % LIU     . MC . max 200 . Liu et al. CVPR 2007      . All
    'SVO'        % SVO     . MC . orig    . Chang et al. ICCV 2011    . All
    'MSSS'       % MSSS    . M  . orig    . Achanta ICIP 2008         . All
    'AC'         % AC      . M  . orig    . Achanta ICVS 2008         . All
    'FTS'        % FT      . M  . orig    . Achanta CVPR 2009         . All
    'MichalGazit'% GAZIT   . M  . orig    . Gazit et.al ECCV2010      . All
    % some precomputed models
    %%%%%%'CA' % ASD only- This is Gofferman, Just keep this.
    %%%%%%'RCC' % Thresholded version of RC, ignore this
    'MZ'         % MZ      . S  . orig    . Ma and Zhang 2003         . ASD only
    'Valenti'    % VALENTI . S  . orig    . Valenti et.al ICCV 2009   . ASD and MSRA only
    'DAKlein'    % BITS    . S  . orig    . Klein & Frintrop ICCV 2011. All
    'PMehrani'   % MV      . S  . orig    . Mehrani et.al BMVC 2010   . ASD and SED1 only
    'LiuICIP'    % LIU-ICIP. S  . orig    . Liu et.al ICIP 2010       . ASD only
    'LiuIETIP'   % LIU-TiP . S  . orig    . Liu et.al TiP 2011        . ASD only
    'MengW'      % MENG    . S  . orig    .                           . ASD only
    %%precomputed on windoze machine
    %%%%% 'FT' % = FTS, ignore this model
    %%%%% 'SR' % This is the same as HouCVPR
    'HC'         % HC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    'LC'         % LC      . C  . orig    . Zhai and Shah ACM-M 2006  . All
    'RC'         % RC      . C  . orig    . Meng Wang et.al CVPR 2011 . All
    %partial result, maybe biased due to train&test on same dataset but different files
    'JiaLiSal'   % JIA-LI  . S  . 1/16    . Jia Li et.al IJCV 2010    . Partial ASD and MSRA
    
    
    % Saliency modeling models (for fixation prediction)
    'SAIM'       % AIM     . M  . 1/2     .
    'SHouCVPR'   % SR      . M  . 64x64   . SRM % SKIP This as it is the same as SR
    'SHouNIPS'   % ICL     . M  . orig    .
    'SSUN'       % SUN     . M  . 1/2 - N .
    'SAWS'       % AWS     . M  . 1/2     .
    'SPQFT'      % PQFT    . M  . 400x400 .
    'SGBVS'      % GBVS    . M  . orig    .have problem when the image is under 128x128, same as GB
    'SITTI'      % ITTI    . C  . 1/16    . Itti model - same as IT
    'SITTI98'    % ITTI98  . C  . 1/16    . Maxnorm
    'SSEO'       % SDSR    . M  . orig    .
    'SJudd'      % Judd    . M  . orig    .
    
    
    % MAP map (Mean Annotation Map)
    'MEP' % Mean Annotation Map . 500x500
    %'IO'  % >>> Inter observer  . orig
    
    % segmenters, not salient object detections
    'SRS1' % SRS . Christian's model, SRS# where # is the #-th salient point
    %     'SRS2'
    %     'SRS3'
    %     'SRS4'
    %     'SRS5'
    'Mishra' % MISHRA .
    
    % some combination maps
    'CMAPmlin' % mult linear
    'CMAPplin' % plus linear
    'CMAPplog' % plus log(x/2)
    'CMAPpexp' % plus exp(x^3)

    };

% permutation
combiList = nchoosek(1:length(modelUsed),2);

% loop dataset
fig1=figure;
for ds=1:length(datasetUsed)
    eval(['imName = ' datasetUsed{ds} 'getList;'])
    
    
    ccModel = zeros(length(modelUsed), length(modelUsed),length(imName));
    % loop image
    for imNo = 1:length(imName)
        eval(['[imBox imEachGT imAllGT] = ' datasetUsed{ds} 'getBox(imName{imNo});'])
        printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName))
        
        % loop models
        imgSM = cell(length(modelUsed),1);
        for mm=1:length(modelUsed)
            if strcmp(modelUsed{mm},'MEP')
                theName = [myPath 'output/MEP_' datasetUsed{ds} '.mat'];
            else
                theName = [myPath 'output/' modelUsed{mm} '/' datasetUsed{ds} '/' rmExt(imName{imNo}) '.mat'];
            end
            if exist(theName,'file')
                load(theName);
                %printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm})
                % fix stuffs
                SM(isnan(SM))=0;
                SM = double(SM);
                if size(SM,3)==3; SM=rgb2gray(SM); end
                SM = imresize(SM,size(imAllGT),'nearest');
                SM = SM  - min(min(SM));
                SM = SM ./ max(max(SM));
                imgSM{mm} = SM;
            else
                printLog(myTic,'dataset',datasetUsed{ds},'| imgNo',imNo,'/',length(imName),'| model',modelUsed{mm},'not found')
            end
        end
        
        % compute cc here
        for mm=1:length(combiList)
            m1=combiList(mm,1);
            m2=combiList(mm,2);
            if isempty(imgSM{m1}) || isempty(imgSM{m2});
                ccModel(m1,m2,imNo) = nan;
                ccModel(m2,m1,imNo) = nan;
            else
                ccModel(m1,m2,imNo) = corr2(imgSM{m1}, imgSM{m2});
                ccModel(m2,m1,imNo) = ccModel(m1,m2,imNo);
            end
        end
    end
    
    ccTotal = zeros(length(modelUsed), length(modelUsed));
    for mm=1:length(combiList)
        m1=combiList(mm,1);
        m2=combiList(mm,2);
        tmp = permute(ccModel(m1,m2,:),[3 1 2]);
        idx = ~isnan(tmp);
        if max(idx);
            tmp = mean(tmp(idx));
            ccTotal(m1,m2)=tmp;
            ccTotal(m2,m1)=tmp;
        end
    end
    save(['score/ccScore_' datasetUsed{ds} '_' date '.mat'],'ccTotal','modelUsed')
    %%% removing all failed
    aa=sum(ccTotal);
    bb=aa~=0;
    
    ccTMP=ccTotal;
    ccTMP=ccTMP(bb,:);
    ccTMP=ccTMP(:,bb);
    ccTMP(ccTMP==0)=1;
    imagesc(ccTMP);
    colorbar
    set(gca,'ytick',1:sum(bb));
    set(gca,'yticklabel',modelUsed(bb));
    colorbar
    set(gca,'xtick',1:length(bb));
    %set(gca,'xticklabel',nn);
    saveas(fig1,['score/ccScore_' datasetUsed{ds} '_' date '__4.fig']);
    
    ccTotal = 1-ccTotal; % 1-cc
    for mm=1:length(modelUsed) % fixing to be 0 on diagonal
        ccTotal(mm,mm)=0;
    end
    ccTotal=ccTotal(bb,:);
    ccTotal=ccTotal(:,bb);
    
    [Y,eig] = cmdscale(ccTotal);
    %%% Plotting
    
    plot(Y(:,1),Y(:,2),'.')
    text(Y(:,1),Y(:,2),modelUsed(bb))
    saveas(fig1,['score/ccScore_' datasetUsed{ds} '_' date '__1.fig']);
    AA=eig(eig>0);
    AA=AA/sum(AA);
    AA=AA(AA>0);
    AA=cumsum(AA);
    plot(AA,'-o','markersize',5)
    %figure
    saveas(fig1,['score/ccScore_' datasetUsed{ds} '_' date '__2.fig']);
    plot3(Y(:,1),Y(:,2),Y(:,3),'.')
    text(Y(:,1),Y(:,2),Y(:,3),modelUsed(bb))
    grid on
    saveas(fig1,['score/ccScore_' datasetUsed{ds} '_' date '__3.fig']);
    

end