function [x_sal, x_obj, init_obj, wiao_obj, boxes, ccScore] = InterSalObj(segment)
% x_sal: estimated saliency for each superpixel
% x_obj: estimated objectness
% init_obj: wiao_obj  without saliency factor
% wiao_obj: objectness from "what is an object?", CVPR 2010.
% boxes: the bounding boxes of the possible objects
% ccScore: object-level saliency

img = imread(segment.imgName);
[h,w,c] = size(img);
if(c==1)
    c = 3;
    img = reshape(repmat(img(:), [1 c]), [h w c]);
end
[~, allboxes] = runObjectness(img,100000);
% remove zero size box
allboxes = DropZeroSizeWin(allboxes);

% remove saliency factor
allboxes = [allboxes, ComputeScore_ID(img, [], [], [], allboxes)];
nMaxSample = 1000;  % You can use a smaller sample size for a faster running time
indexSamples = scoreSampling(allboxes(:,end),nMaxSample,1); 
boxes = allboxes(indexSamples,:);

% record wiao_obj, init_obj
allboxes = boxes;
allboxes = DropRepWin(allboxes);
wiao_obj = allboxes(:,5);
init_obj = allboxes(:,6);
allboxes = allboxes(:,[1:4,6]);

score_td = allboxes(:,5);
% score_td = ones(length(allboxes(:,5)), 1)*.5;   % uniform prior - very bad

% fixed variables
nSample = size(allboxes, 1);
c_4 = 32;
c_1 = 64/c_4; c_2 = 16000/(c_4*nSample); c_3 = 1/c_4;
% c_1 = 64/c_4; c_2 = 1.6/c_4; c_3 = 1/c_4;  % 10000 sample
% c_1 = 64/c_4; c_2 = 16/c_4; c_3 = 1/c_4;  % 1000 sample

lMatrix = LapWith2Iden(segment, c_1, c_3);
lMatrixInv = inv(lMatrix);
% low level saliency:
spSalPrior = SPSalPrior(segment);

spSalPrior = 1./(1+exp(-spSalPrior*2));
spSalPrior = (spSalPrior-.5)/.5;

% object-level saliency:
% [spInWin spToCC ccScore] = SPInWin(segment, allboxes);   % window-based center-surround
[spInWin spToCC ccScore] = SPInWinV1(segment, allboxes);   % object-based center-surround

% init. x_obj
x_obj = allboxes(:,5);
x_objOld = x_obj;
% init. x_sal

% map = SPSalMap(segment, spSalPrior);
% figure; subplot('position', [0,0,1,1]); imagesc(map);

x_sal = spSalPrior;
% x_sal(x_sal<.3) = 0;
x_salOld = x_sal;

engOld = ComputeEng(lMatrix, spSalPrior, score_td, spToCC, spInWin, x_sal, x_obj, c_1, c_2, 0);
% fprintf('init eng:%10.4f\n', engOld);
% [~,scOrder] = sort(x_obj, 'descend');
% figure; subplot('position', [0,0,1,1]); imagesc(img);
% hold on; drawBoxes(allboxes(scOrder(1:10),:));
% map = SPSalMap(segment, x_sal);
% figure; subplot('position', [0,0,1,1]); imagesc(map);

for k=1:100
    % map = SPSalMap(segment, x_sal);
    % figure; subplot('position', [0,0,1,1]); imagesc(map);
%     score_id = ImgDepScore_SP(x_sal, segment, .5, spInWin, allboxes);
%     x_obj = FindX_Obj(allboxes(:,5), score_id);
%     tic;
    x_obj = FindX_Obj(score_td, spToCC, spInWin, x_sal, x_obj, c_2);
%     toc;
    x_sal = FindX_Sal(lMatrixInv, spSalPrior, spToCC, spInWin, x_obj, c_1);

    eng = ComputeEng(lMatrix, spSalPrior, score_td, spToCC, spInWin, x_sal, x_obj, c_1, c_2, 0);
%     fprintf('eng:%10.4f\n', eng);
    if((max(abs(x_salOld-x_sal))<.001)||(eng>=engOld))
        %converge
%         fprintf('converge!!, iter:%2d.\n', k);
        
        eng = engOld;
        x_sal = x_salOld;
        x_obj = x_objOld;
        break;
    else
        if(eng<engOld)
            engOld = eng;
        end
        x_salOld = x_sal;
        x_objOld = x_obj;
    end
    
    % compute score_id
%     score_td = ComputeScore_ID(img, x_sal, segment, spInWin, allboxes);
    
% [~,scOrder] = sort(score_td, 'descend');
% figure; subplot('position', [0,0,1,1]); imagesc(img);
% hold on; drawBoxes(allboxes(scOrder(1:10),:));

%     [~,scOrder] = sort(x_obj, 'descend');
%     figure; subplot('position', [0,0,1,1]); imagesc(img);
%     hold on; drawBoxes(allboxes(scOrder(1:10),:));
%     map = SPSalMap(segment, x_sal);
%     figure; subplot('position', [0,0,1,1]); imagesc(map);
end


% [~,scOrder] = sort(x_obj, 'descend');
% figure; subplot('position', [0,0,1,1]); imagesc(img);
% hold on; drawBoxes(allboxes(scOrder(1:10),:));
% map = SPSalMap(segment, x_sal);
% figure; subplot('position', [0,0,1,1]); imagesc(map);

boxes = allboxes;