function simMatrix = SimMatrix(segment)
% affinity matrix
img = im2double(imread(segment.imgName));
[h,w,c] = size(img);
if(c==1)
    c = 3;
    img = reshape(repmat(img(:), [1 c]), [h w c]);
end
[pairEng pairSeg] = SubSPPairTerm(segment, img, 20/256);

maxEng = max(pairEng);
simMatrix = zeros(segment.stat.P,segment.stat.P);
for k=1:segment.stat.E
    simMatrix(pairSeg(k,1),pairSeg(k,2)) = pairEng(k);
    simMatrix(pairSeg(k,2),pairSeg(k,1)) = pairEng(k);
end

% not necessary for laplacian matrix
% for k=1:segment.stat.P
%     simMatrix(k,k) = maxEng;
% end

return;

function [pairEng pairSeg] = SubSPPairTerm(segment, img, smoothSigma)
% img: rgb, double, [0,1] image
% smoothSigma: default=20/256
[dxl1, dxr1, dyt1, dyb1] = Grad(img(:,:,1));
[dxl2, dxr2, dyt2, dyb2] = Grad(img(:,:,2));
[dxl3, dxr3, dyt3, dyb3] = Grad(img(:,:,3));
dxl = exp(-(dxl1.^2 + dxl2.^2 + dxl3.^2)/(2*smoothSigma*smoothSigma));
dxr = exp(-(dxr1.^2 + dxr2.^2 + dxr3.^2)/(2*smoothSigma*smoothSigma));
dyt = exp(-(dyt1.^2 + dyt2.^2 + dyt3.^2)/(2*smoothSigma*smoothSigma));
dyb = exp(-(dyb1.^2 + dyb2.^2 + dyb3.^2)/(2*smoothSigma*smoothSigma));

pairSeg = segment.pairSeg;

seg = segment.seg;
[dSegxl dSegxr dSegyt dSegyb] = Grad(seg);
dSegxl = int32(dSegxl); dSegxr = int32(dSegxr); dSegyt = int32(dSegyt); dSegyb = int32(dSegyb);
for nSegIdx = 1:segment.stat.P
    segMask = (seg==nSegIdx);

    dSegxl(segMask&(dSegxl~=0)) = dSegxl(segMask&(dSegxl~=0))+nSegIdx;
    dSegxr(segMask&(dSegxr~=0)) = dSegxr(segMask&(dSegxr~=0))+nSegIdx;
    dSegyt(segMask&(dSegyt~=0)) = dSegyt(segMask&(dSegyt~=0))+nSegIdx;
    dSegyb(segMask&(dSegyb~=0)) = dSegyb(segMask&(dSegyb~=0))+nSegIdx;
end

pairEng = zeros(segment.stat.E, 1);
for k=1:segment.stat.E
    tmpPairEng = 0;
    segMask = (seg==pairSeg(k,1));
    tmp = dxl(segMask); tmpPairEng = tmpPairEng + sum(tmp(dSegxl(segMask)==pairSeg(k,2)));
    tmp = dxr(segMask); tmpPairEng = tmpPairEng + sum(tmp(dSegxr(segMask)==pairSeg(k,2)));
    tmp = dyt(segMask); tmpPairEng = tmpPairEng + sum(tmp(dSegyt(segMask)==pairSeg(k,2)));
    tmp = dyb(segMask); tmpPairEng = tmpPairEng + sum(tmp(dSegyb(segMask)==pairSeg(k,2)));
    
    segMask = (seg==pairSeg(k,2));
    tmp = dxl(segMask); tmpPairEng = tmpPairEng + sum(tmp(dSegxl(segMask)==pairSeg(k,1)));
    tmp = dxr(segMask); tmpPairEng = tmpPairEng + sum(tmp(dSegxr(segMask)==pairSeg(k,1)));
    tmp = dyt(segMask); tmpPairEng = tmpPairEng + sum(tmp(dSegyt(segMask)==pairSeg(k,1)));
    tmp = dyb(segMask); tmpPairEng = tmpPairEng + sum(tmp(dSegyb(segMask)==pairSeg(k,1)));
    
    pairEng(k) = tmpPairEng/2 * 4;
end

return;


function [dxl dxr dyt dyb] = Grad(seg)

[h w] = size(seg);

dxl = zeros(h, w);   dxl(:, 2:w) = seg(:, 1:(w-1)) - seg(:, 2:w);
dxr = zeros(h, w);   dxr(:, 1:(w-1)) = seg(:, 2:w) - seg(:, 1:(w-1));
dyt = zeros(h, w);   dyt(2:h, :) = seg(1:(h-1), :) - seg(2:h, :);
dyb = zeros(h, w);   dyb(1:(h-1), :) = seg(2:h, :) - seg(1:(h-1), :);
